
from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from ntnx_security_py_client.api.approval_policies_api import ApprovalPoliciesApi
from ntnx_security_py_client.api.credentials_api import CredentialsApi
from ntnx_security_py_client.api.key_management_servers_api import KeyManagementServersApi
from ntnx_security_py_client.api.sti_gs_api import STIGsApi



# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_security_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.Severity import Severity  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.StigStatus import StigStatus  # noqa: F401,E501

class Stig(ExternalizableAbstractModel):

    """Contains information of a specific STIG.

    :param title: (:attr:`title`) Title of the STIG control.
    :type title: 
     max=199 chars
    
    :param rule_id: (:attr:`rule_id`) Rule ID of the STIG control.
    :type rule_id: 
    
    
    :param stig_version: (:attr:`stig_version`) STIG ID of the control.
    :type stig_version: 
    
    
    :param identifiers: (:attr:`identifiers`) Additional identifiers used to describe this control.
    :type identifiers: 
    
    
    :param affected_clusters: (:attr:`affected_clusters`) List of clusters that failed the STIG control.
    :type affected_clusters: 
    
    
    :param severity: (:attr:`severity`) 
    :type severity: 
    
    
    :param status: (:attr:`status`) 
    :type status: 
    
    
    :param comments: (:attr:`comments`) The comments to explain why a STIG rule applies or does not apply to the cluster.
    :type comments: 
     max=499 chars
    
    :param fix_text: (:attr:`fix_text`) The command/steps to fix the STIG rule failure.
    :type fix_text: 
     max=499 chars
    
    :param benchmark_id: (:attr:`benchmark_id`) Benchmark ID of the STIG rules.
    :type benchmark_id: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'title': 'str',
        'rule_id': 'str',
        'stig_version': 'str',
        'identifiers': 'list[str]',
        'affected_clusters': 'list[str]',
        'severity': 'security.v4.report.Severity',
        'status': 'security.v4.report.StigStatus',
        'comments': 'str',
        'fix_text': 'str',
        'benchmark_id': 'str',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'title': 'title',
        'rule_id': 'ruleId',
        'stig_version': 'stigVersion',
        'identifiers': 'identifiers',
        'affected_clusters': 'affectedClusters',
        'severity': 'severity',
        'status': 'status',
        'comments': 'comments',
        'fix_text': 'fixText',
        'benchmark_id': 'benchmarkId',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, title=None, rule_id=None, stig_version=None, identifiers=None, affected_clusters=None, severity=None, status=None, comments=None, fix_text=None, benchmark_id=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__title = None
        self.__rule_id = None
        self.__stig_version = None
        self.__identifiers = None
        self.__affected_clusters = None
        self.__severity = None
        self.__status = None
        self.__comments = None
        self.__fix_text = None
        self.__benchmark_id = None
        self.discriminator = None
        if title is not None:
            self.__title = title
        if rule_id is not None:
            self.__rule_id = rule_id
        if stig_version is not None:
            self.__stig_version = stig_version
        if identifiers is not None:
            self.__identifiers = identifiers
        if affected_clusters is not None:
            self.__affected_clusters = affected_clusters
        if severity is not None:
            self.__severity = severity
        if status is not None:
            self.__status = status
        if comments is not None:
            self.__comments = comments
        if fix_text is not None:
            self.__fix_text = fix_text
        if benchmark_id is not None:
            self.__benchmark_id = benchmark_id

    def _initialize_object_type(self):
        return 'security.v4.report.Stig'

    def _initialize_object_version(self):
        return 'v4.r0'


    @property
    def title(self):
        """
        Title of the STIG control.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__title

    @title.setter
    def title(self, title):
        if title is not None and len(title) > 199:
            raise ValueError("Invalid value for `title`, length must be less than or equal to `199`")  # noqa: E501

        self.__title = title

    @property
    def rule_id(self):
        """
        Rule ID of the STIG control.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__rule_id

    @rule_id.setter
    def rule_id(self, rule_id):
        if rule_id is not None and not re.search('^SV-\\d{6}r\\d{6}_rule$', rule_id):  # noqa: E501
            raise ValueError(r"Invalid value for `rule_id`, must be a follow pattern or equal to `/^SV-\\d{6}r\\d{6}_rule$/`")  # noqa: E501

        self.__rule_id = rule_id

    @property
    def stig_version(self):
        """
        STIG ID of the control.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__stig_version

    @stig_version.setter
    def stig_version(self, stig_version):
        if stig_version is not None and not re.search('^RHEL-\\d{2}-\\d{6}$', stig_version):  # noqa: E501
            raise ValueError(r"Invalid value for `stig_version`, must be a follow pattern or equal to `/^RHEL-\\d{2}-\\d{6}$/`")  # noqa: E501

        self.__stig_version = stig_version

    @property
    def identifiers(self):
        """
        Additional identifiers used to describe this control.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__identifiers

    @identifiers.setter
    def identifiers(self, identifiers):

        self.__identifiers = identifiers

    @property
    def affected_clusters(self):
        """
        List of clusters that failed the STIG control.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__affected_clusters

    @affected_clusters.setter
    def affected_clusters(self, affected_clusters):

        self.__affected_clusters = affected_clusters

    @property
    def severity(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.security.v4.report.Severity`
        """  # noqa: E501
        return self.__severity

    @severity.setter
    def severity(self, severity):

        self.__severity = severity

    @property
    def status(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.security.v4.report.StigStatus`
        """  # noqa: E501
        return self.__status

    @status.setter
    def status(self, status):

        self.__status = status

    @property
    def comments(self):
        """
        The comments to explain why a STIG rule applies or does not apply to the cluster.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__comments

    @comments.setter
    def comments(self, comments):
        if comments is not None and len(comments) > 499:
            raise ValueError("Invalid value for `comments`, length must be less than or equal to `499`")  # noqa: E501

        self.__comments = comments

    @property
    def fix_text(self):
        """
        The command/steps to fix the STIG rule failure.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__fix_text

    @fix_text.setter
    def fix_text(self, fix_text):
        if fix_text is not None and len(fix_text) > 499:
            raise ValueError("Invalid value for `fix_text`, length must be less than or equal to `499`")  # noqa: E501

        self.__fix_text = fix_text

    @property
    def benchmark_id(self):
        """
        Benchmark ID of the STIG rules.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__benchmark_id

    @benchmark_id.setter
    def benchmark_id(self, benchmark_id):
        if benchmark_id is not None and not re.search('^RHEL_\\d+_V\\d+R\\d+$', benchmark_id):  # noqa: E501
            raise ValueError(r"Invalid value for `benchmark_id`, must be a follow pattern or equal to `/^RHEL_\\d+_V\\d+R\\d+$/`")  # noqa: E501

        self.__benchmark_id = benchmark_id

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(Stig, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Stig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.common.v1.config.KVPair import KVPair  # noqa: F401,E501
from ntnx_security_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_security_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.BucketsAccessKey import BucketsAccessKey  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.CreationType import CreationType  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.UserStatusType import UserStatusType  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.UserType import UserType  # noqa: F401,E501

class User(ExternalizableAbstractModel):

    """Information of the user.

    :param username: (:attr:`username`) Identifier of the user.
    :type username: 
     min=1 chars, max=255 chars
    
    :param user_type: (:attr:`user_type`) 
    :type user_type: 
    
    
    :param idp_id: (:attr:`idp_id`) Identifier of the IDP for the user.
    :type idp_id: 
    
    
    :param display_name: (:attr:`display_name`) Display name of the user. For LDAP and SAML users, this is set from AD config.
    :type display_name: 
     min=1 chars, max=255 chars
    
    :param first_name: (:attr:`first_name`) First name of the user.
    :type first_name: 
     min=1 chars, max=255 chars
    
    :param middle_initial: (:attr:`middle_initial`) Middle name of the user.
    :type middle_initial: 
     min=1 chars, max=255 chars
    
    :param last_name: (:attr:`last_name`) Last name of the user.
    :type last_name: 
     min=1 chars, max=255 chars
    
    :param email_id: (:attr:`email_id`) Email ID of the user.
    :type email_id: 
     max=255 chars
    
    :param locale: (:attr:`locale`) Default locale of the user.
    :type locale: 
    
    
    :param region: (:attr:`region`) Default region of the user.
    :type region: 
    
    
    :param password: (:attr:`password`) Password of the user.
    :type password: 
    
    
    :param is_force_reset_password_enabled: (:attr:`is_force_reset_password_enabled`) Flag to force the user to reset password.
    :type is_force_reset_password_enabled: 
    
    
    :param additional_attributes: (:attr:`additional_attributes`) Indicates additional attributes of the user.
    :type additional_attributes: 
    
    
    :param status: (:attr:`status`) 
    :type status: 
    
    
    :param buckets_access_keys: (:attr:`buckets_access_keys`) Bucket access keys for the user.
    :type buckets_access_keys: 
    
    
    :param last_login_time: (:attr:`last_login_time`) The last successful login time for the user.
    :type last_login_time: 
    
    
    :param created_time: (:attr:`created_time`) Creation time of the user.
    :type created_time: 
    
    
    :param last_updated_time: (:attr:`last_updated_time`) The last updated time for the user.
    :type last_updated_time: 
    
    
    :param created_by: (:attr:`created_by`) User or Service who created the user.
    :type created_by: 
    
    
    :param last_updated_by: (:attr:`last_updated_by`) Last updated by this user ID.
    :type last_updated_by: 
    
    
    :param description: (:attr:`description`) Description of the user.
    :type description: 
    
    
    :param creation_type: (:attr:`creation_type`) 
    :type creation_type: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'username': 'str',
        'user_type': 'iam.v4.authn.UserType',
        'idp_id': 'str',
        'display_name': 'str',
        'first_name': 'str',
        'middle_initial': 'str',
        'last_name': 'str',
        'email_id': 'str',
        'locale': 'str',
        'region': 'str',
        'password': 'str',
        'is_force_reset_password_enabled': 'bool',
        'additional_attributes': 'list[common.v1.config.KVPair]',
        'status': 'iam.v4.authn.UserStatusType',
        'buckets_access_keys': 'list[iam.v4.authn.BucketsAccessKey]',
        'last_login_time': 'datetime',
        'created_time': 'datetime',
        'last_updated_time': 'datetime',
        'created_by': 'str',
        'last_updated_by': 'str',
        'description': 'str',
        'creation_type': 'iam.v4.authn.CreationType',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'username': 'username',
        'user_type': 'userType',
        'idp_id': 'idpId',
        'display_name': 'displayName',
        'first_name': 'firstName',
        'middle_initial': 'middleInitial',
        'last_name': 'lastName',
        'email_id': 'emailId',
        'locale': 'locale',
        'region': 'region',
        'password': 'password',
        'is_force_reset_password_enabled': 'isForceResetPasswordEnabled',
        'additional_attributes': 'additionalAttributes',
        'status': 'status',
        'buckets_access_keys': 'bucketsAccessKeys',
        'last_login_time': 'lastLoginTime',
        'created_time': 'createdTime',
        'last_updated_time': 'lastUpdatedTime',
        'created_by': 'createdBy',
        'last_updated_by': 'lastUpdatedBy',
        'description': 'description',
        'creation_type': 'creationType',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, username=None, user_type=None, idp_id=None, display_name=None, first_name=None, middle_initial=None, last_name=None, email_id=None, locale=None, region=None, password=None, is_force_reset_password_enabled=None, additional_attributes=None, status=None, buckets_access_keys=None, last_login_time=None, created_time=None, last_updated_time=None, created_by=None, last_updated_by=None, description=None, creation_type=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__username = None
        self.__user_type = None
        self.__idp_id = None
        self.__display_name = None
        self.__first_name = None
        self.__middle_initial = None
        self.__last_name = None
        self.__email_id = None
        self.__locale = None
        self.__region = None
        self.__password = None
        self.__is_force_reset_password_enabled = None
        self.__additional_attributes = None
        self.__status = None
        self.__buckets_access_keys = None
        self.__last_login_time = None
        self.__created_time = None
        self.__last_updated_time = None
        self.__created_by = None
        self.__last_updated_by = None
        self.__description = None
        self.__creation_type = None
        self.discriminator = None
        if username is not None:
            self.__username = username
        if user_type is not None:
            self.__user_type = user_type
        if idp_id is not None:
            self.__idp_id = idp_id
        if display_name is not None:
            self.__display_name = display_name
        if first_name is not None:
            self.__first_name = first_name
        if middle_initial is not None:
            self.__middle_initial = middle_initial
        if last_name is not None:
            self.__last_name = last_name
        if email_id is not None:
            self.__email_id = email_id
        if locale is not None:
            self.__locale = locale
        if region is not None:
            self.__region = region
        if password is not None:
            self.__password = password
        if is_force_reset_password_enabled is not None:
            self.__is_force_reset_password_enabled = is_force_reset_password_enabled
        if additional_attributes is not None:
            self.__additional_attributes = additional_attributes
        if status is not None:
            self.__status = status
        if buckets_access_keys is not None:
            self.__buckets_access_keys = buckets_access_keys
        if last_login_time is not None:
            self.__last_login_time = last_login_time
        if created_time is not None:
            self.__created_time = created_time
        if last_updated_time is not None:
            self.__last_updated_time = last_updated_time
        if created_by is not None:
            self.__created_by = created_by
        if last_updated_by is not None:
            self.__last_updated_by = last_updated_by
        if description is not None:
            self.__description = description
        if creation_type is not None:
            self.__creation_type = creation_type

    def _initialize_object_type(self):
        return 'iam.v4.authn.User'

    def _initialize_object_version(self):
        return 'v4.r1.b1'


    @property
    def username(self):
        """
        Identifier of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__username

    @username.setter
    def username(self, username):
        if username is not None and len(username) > 255:
            raise ValueError("Invalid value for `username`, length must be less than or equal to `255`")  # noqa: E501
        if username is not None and len(username) < 1:
            raise ValueError("Invalid value for `username`, length must be greater than or equal to `1`")  # noqa: E501
        if username is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', username):  # noqa: E501
            raise ValueError(r"Invalid value for `username`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__username = username

    @property
    def user_type(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.iam.v4.authn.UserType`
        """  # noqa: E501
        return self.__user_type

    @user_type.setter
    def user_type(self, user_type):

        self.__user_type = user_type

    @property
    def idp_id(self):
        """
        Identifier of the IDP for the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__idp_id

    @idp_id.setter
    def idp_id(self, idp_id):
        if idp_id is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', idp_id):  # noqa: E501
            raise ValueError(r"Invalid value for `idp_id`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__idp_id = idp_id

    @property
    def display_name(self):
        """
        Display name of the user. For LDAP and SAML users, this is set from AD config.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__display_name

    @display_name.setter
    def display_name(self, display_name):
        if display_name is not None and len(display_name) > 255:
            raise ValueError("Invalid value for `display_name`, length must be less than or equal to `255`")  # noqa: E501
        if display_name is not None and len(display_name) < 1:
            raise ValueError("Invalid value for `display_name`, length must be greater than or equal to `1`")  # noqa: E501
        if display_name is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', display_name):  # noqa: E501
            raise ValueError(r"Invalid value for `display_name`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__display_name = display_name

    @property
    def first_name(self):
        """
        First name of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__first_name

    @first_name.setter
    def first_name(self, first_name):
        if first_name is not None and len(first_name) > 255:
            raise ValueError("Invalid value for `first_name`, length must be less than or equal to `255`")  # noqa: E501
        if first_name is not None and len(first_name) < 1:
            raise ValueError("Invalid value for `first_name`, length must be greater than or equal to `1`")  # noqa: E501
        if first_name is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', first_name):  # noqa: E501
            raise ValueError(r"Invalid value for `first_name`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__first_name = first_name

    @property
    def middle_initial(self):
        """
        Middle name of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__middle_initial

    @middle_initial.setter
    def middle_initial(self, middle_initial):
        if middle_initial is not None and len(middle_initial) > 255:
            raise ValueError("Invalid value for `middle_initial`, length must be less than or equal to `255`")  # noqa: E501
        if middle_initial is not None and len(middle_initial) < 1:
            raise ValueError("Invalid value for `middle_initial`, length must be greater than or equal to `1`")  # noqa: E501
        if middle_initial is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', middle_initial):  # noqa: E501
            raise ValueError(r"Invalid value for `middle_initial`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__middle_initial = middle_initial

    @property
    def last_name(self):
        """
        Last name of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__last_name

    @last_name.setter
    def last_name(self, last_name):
        if last_name is not None and len(last_name) > 255:
            raise ValueError("Invalid value for `last_name`, length must be less than or equal to `255`")  # noqa: E501
        if last_name is not None and len(last_name) < 1:
            raise ValueError("Invalid value for `last_name`, length must be greater than or equal to `1`")  # noqa: E501
        if last_name is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', last_name):  # noqa: E501
            raise ValueError(r"Invalid value for `last_name`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__last_name = last_name

    @property
    def email_id(self):
        """
        Email ID of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__email_id

    @email_id.setter
    def email_id(self, email_id):
        if email_id is not None and len(email_id) > 255:
            raise ValueError("Invalid value for `email_id`, length must be less than or equal to `255`")  # noqa: E501
        if email_id is not None and not re.search('^([a-zA-Z0-9_\\-\\.]+)@([a-zA-Z0-9_\\-\\.]+)\\.([a-zA-Z]{2,5})$', email_id):  # noqa: E501
            raise ValueError(r"Invalid value for `email_id`, must be a follow pattern or equal to `/^([a-zA-Z0-9_\\-\\.]+)@([a-zA-Z0-9_\\-\\.]+)\\.([a-zA-Z]{2,5})$/`")  # noqa: E501

        self.__email_id = email_id

    @property
    def locale(self):
        """
        Default locale of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__locale

    @locale.setter
    def locale(self, locale):

        self.__locale = locale

    @property
    def region(self):
        """
        Default region of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__region

    @region.setter
    def region(self, region):

        self.__region = region

    @property
    def password(self):
        """
        Password of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__password

    @password.setter
    def password(self, password):

        self.__password = password

    @property
    def is_force_reset_password_enabled(self):
        """
        Flag to force the user to reset password.

        :type:
                :class:`~bool`
        """  # noqa: E501
        return self.__is_force_reset_password_enabled

    @is_force_reset_password_enabled.setter
    def is_force_reset_password_enabled(self, is_force_reset_password_enabled):

        self.__is_force_reset_password_enabled = is_force_reset_password_enabled

    @property
    def additional_attributes(self):
        """
        Indicates additional attributes of the user.

        :type:
             list[ :class:`~ntnx_security_py_client.models.common.v1.config.KVPair` ]
        """  # noqa: E501
        return self.__additional_attributes

    @additional_attributes.setter
    def additional_attributes(self, additional_attributes):

        self.__additional_attributes = additional_attributes

    @property
    def status(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.iam.v4.authn.UserStatusType`
        """  # noqa: E501
        return self.__status

    @status.setter
    def status(self, status):

        self.__status = status

    @property
    def buckets_access_keys(self):
        """
        Bucket access keys for the user.

        :type:
             list[ :class:`~ntnx_security_py_client.models.iam.v4.authn.BucketsAccessKey` ]
        """  # noqa: E501
        return self.__buckets_access_keys

    @buckets_access_keys.setter
    def buckets_access_keys(self, buckets_access_keys):

        self.__buckets_access_keys = buckets_access_keys

    @property
    def last_login_time(self):
        """
        The last successful login time for the user.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__last_login_time

    @last_login_time.setter
    def last_login_time(self, last_login_time):

        self.__last_login_time = last_login_time

    @property
    def created_time(self):
        """
        Creation time of the user.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__created_time

    @created_time.setter
    def created_time(self, created_time):

        self.__created_time = created_time

    @property
    def last_updated_time(self):
        """
        The last updated time for the user.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__last_updated_time

    @last_updated_time.setter
    def last_updated_time(self, last_updated_time):

        self.__last_updated_time = last_updated_time

    @property
    def created_by(self):
        """
        User or Service who created the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__created_by

    @created_by.setter
    def created_by(self, created_by):
        if created_by is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', created_by):  # noqa: E501
            raise ValueError(r"Invalid value for `created_by`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__created_by = created_by

    @property
    def last_updated_by(self):
        """
        Last updated by this user ID.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        if last_updated_by is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', last_updated_by):  # noqa: E501
            raise ValueError(r"Invalid value for `last_updated_by`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__last_updated_by = last_updated_by

    @property
    def description(self):
        """
        Description of the user.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__description

    @description.setter
    def description(self, description):

        self.__description = description

    @property
    def creation_type(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.iam.v4.authn.CreationType`
        """  # noqa: E501
        return self.__creation_type

    @creation_type.setter
    def creation_type(self, creation_type):

        self.__creation_type = creation_type

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(User, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, User):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from ntnx_security_py_client.api_client import ApiClient


class KeyManagementServersApi(object):
    """IGNORE:
    NOTE: A placeholder for class level description
    IGNORE
    """  # noqa: E501

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()

        self.api_client = api_client
        self.__headers_to_skip = { 'authorization', 'cookie', 'host', 'user-agent' }

    def create_key_management_server(self, body, **kwargs):  # noqa: E501
        """Create a key management server

        >>> response = api.create_key_management_server(body)

        This method makes a synchronous HTTP request by default. To make an asynchronous HTTP request, please pass async_req=True.

        >>> thread = api.create_key_management_server(body, async_req=True)
        >>> result = thread.get()

        :param body: Parameters required to create the key management server.
        :type body:
            :class:`~ntnx_security_py_client.models.security.v4.config.KeyManagementServer`, required
        :param kwargs: kwargs for the method.
                       The following can be passed for an asynchronous HTTP call::

                           async_req=True
        :return: An instance of class :class:`~ntnx_security_py_client.models.security.v4.config.CreateKeyManagementServerApiResponse`.

                 If the method is called asynchronously, returns the request thread.
        """ # noqa: E501
        kwargs['_return_http_data_only'] = True

        params = dict(locals())
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        del params['self']

        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_key_management_server`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client._select_header_accept(
            ['application/json'])  # noqa: E501
        if 'Accept' in params and params.get('Accept') is not None:
            header_params['Accept'] = params.get('Accept')

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client._select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501
        if 'Content-Type' in params and params.get('Content-Type') is not None:
            header_params['Content-Type'] = params.get('Content-Type')

        # Process operation specific headers
        extra_params = []
        extra_params.append('async_req')
        extra_params.append('_return_http_data_only')
        extra_params.append('_preload_content')
        extra_params.append('_request_timeout')
        all_params = set(['body'])
        all_params.update(extra_params)
        for key, val in six.iteritems(params):
            if val is not None and key.lower() not in self.__headers_to_skip and key not in all_params:
                if key.lower() == 'if_match'.lower():
                    key = 'If-Match'
                elif key.lower() == 'if_none_match'.lower():
                    key = 'If-None-Match'
                header_params[key] = val

        form_params = []
        local_var_files = {}

        # Authentication setting
        auth_settings = ['apiKeyAuthScheme', 'basicAuthScheme']  # noqa: E501

        body_params = None
        if 'body' in params and params['body'] is not None:
            body_params = params['body']

        try:
            if kwargs.get('async_req'):
                return self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers', 'POST',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.CreateKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
            else:
                (data) = self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers', 'POST',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.CreateKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
                return data
        finally:
            pass

    def delete_key_management_server_by_id(self, extId, **kwargs):  # noqa: E501
        """Delete a key management server

        >>> response = api.delete_key_management_server_by_id(extId)

        This method makes a synchronous HTTP request by default. To make an asynchronous HTTP request, please pass async_req=True.

        >>> thread = api.delete_key_management_server_by_id(extId, async_req=True)
        >>> result = thread.get()

        :param extId: Unique identifier for the key management server of type UUID.
        :type extId:
            :class:`str`, required
        :param kwargs: kwargs for the method.
                       The following can be passed for an asynchronous HTTP call::

                           async_req=True
        :return: An instance of class :class:`~ntnx_security_py_client.models.security.v4.config.DeleteKeyManagementServerApiResponse`.

                 If the method is called asynchronously, returns the request thread.
        """ # noqa: E501
        kwargs['_return_http_data_only'] = True

        params = dict(locals())
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        del params['self']

        # verify the required parameter 'extId' is set
        if ('extId' not in params or params['extId'] is None):
            raise ValueError("Missing the required parameter `extId` when calling `delete_key_management_server_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'extId' in params and params['extId'] is not None:
            path_params['extId'] = params['extId']  # noqa: E501

        query_params = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client._select_header_accept(
            ['application/json'])  # noqa: E501
        if 'Accept' in params and params.get('Accept') is not None:
            header_params['Accept'] = params.get('Accept')

        # Process operation specific headers
        extra_params = []
        extra_params.append('async_req')
        extra_params.append('_return_http_data_only')
        extra_params.append('_preload_content')
        extra_params.append('_request_timeout')
        all_params = set(['extId'])
        all_params.update(extra_params)
        for key, val in six.iteritems(params):
            if val is not None and key.lower() not in self.__headers_to_skip and key not in all_params:
                if key.lower() == 'if_match'.lower():
                    key = 'If-Match'
                elif key.lower() == 'if_none_match'.lower():
                    key = 'If-None-Match'
                header_params[key] = val

        form_params = []
        local_var_files = {}

        # Authentication setting
        auth_settings = ['apiKeyAuthScheme', 'basicAuthScheme']  # noqa: E501

        body_params = None

        try:
            if kwargs.get('async_req'):
                return self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'DELETE',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.DeleteKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
            else:
                (data) = self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'DELETE',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.DeleteKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
                return data
        finally:
            pass

    def get_key_management_server_by_id(self, extId, **kwargs):  # noqa: E501
        """Get details of a key management server

        >>> response = api.get_key_management_server_by_id(extId)

        This method makes a synchronous HTTP request by default. To make an asynchronous HTTP request, please pass async_req=True.

        >>> thread = api.get_key_management_server_by_id(extId, async_req=True)
        >>> result = thread.get()

        :param extId: Unique identifier for the key management server of type UUID.
        :type extId:
            :class:`str`, required
        :param kwargs: kwargs for the method.
                       The following can be passed for an asynchronous HTTP call::

                           async_req=True
        :return: An instance of class :class:`~ntnx_security_py_client.models.security.v4.config.GetKeyManagementServerApiResponse`.

                 If the method is called asynchronously, returns the request thread.
        """ # noqa: E501
        kwargs['_return_http_data_only'] = True

        params = dict(locals())
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        del params['self']

        # verify the required parameter 'extId' is set
        if ('extId' not in params or params['extId'] is None):
            raise ValueError("Missing the required parameter `extId` when calling `get_key_management_server_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'extId' in params and params['extId'] is not None:
            path_params['extId'] = params['extId']  # noqa: E501

        query_params = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client._select_header_accept(
            ['application/json'])  # noqa: E501
        if 'Accept' in params and params.get('Accept') is not None:
            header_params['Accept'] = params.get('Accept')

        # Process operation specific headers
        extra_params = []
        extra_params.append('async_req')
        extra_params.append('_return_http_data_only')
        extra_params.append('_preload_content')
        extra_params.append('_request_timeout')
        all_params = set(['extId'])
        all_params.update(extra_params)
        for key, val in six.iteritems(params):
            if val is not None and key.lower() not in self.__headers_to_skip and key not in all_params:
                if key.lower() == 'if_match'.lower():
                    key = 'If-Match'
                elif key.lower() == 'if_none_match'.lower():
                    key = 'If-None-Match'
                header_params[key] = val

        form_params = []
        local_var_files = {}

        # Authentication setting
        auth_settings = ['apiKeyAuthScheme', 'basicAuthScheme']  # noqa: E501

        body_params = None

        try:
            if kwargs.get('async_req'):
                return self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'GET',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.GetKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
            else:
                (data) = self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'GET',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.GetKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
                return data
        finally:
            pass

    def list_key_management_servers(self, **kwargs):  # noqa: E501
        """List key management servers

        >>> response = api.list_key_management_servers()

        This method makes a synchronous HTTP request by default. To make an asynchronous HTTP request, please pass async_req=True.

        >>> thread = api.list_key_management_servers(async_req=True)
        >>> result = thread.get()

        :param kwargs: kwargs for the method.
                       The following can be passed for an asynchronous HTTP call::

                           async_req=True
        :return: An instance of class :class:`~ntnx_security_py_client.models.security.v4.config.ListKeyManagementServersApiResponse`.

                 If the method is called asynchronously, returns the request thread.
        """ # noqa: E501
        kwargs['_return_http_data_only'] = True

        params = dict(locals())
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        del params['self']


        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client._select_header_accept(
            ['application/json'])  # noqa: E501
        if 'Accept' in params and params.get('Accept') is not None:
            header_params['Accept'] = params.get('Accept')

        # Process operation specific headers
        extra_params = []
        extra_params.append('async_req')
        extra_params.append('_return_http_data_only')
        extra_params.append('_preload_content')
        extra_params.append('_request_timeout')
        all_params = set([])
        all_params.update(extra_params)
        for key, val in six.iteritems(params):
            if val is not None and key.lower() not in self.__headers_to_skip and key not in all_params:
                if key.lower() == 'if_match'.lower():
                    key = 'If-Match'
                elif key.lower() == 'if_none_match'.lower():
                    key = 'If-None-Match'
                header_params[key] = val

        form_params = []
        local_var_files = {}

        # Authentication setting
        auth_settings = ['apiKeyAuthScheme', 'basicAuthScheme']  # noqa: E501

        body_params = None

        try:
            if kwargs.get('async_req'):
                return self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers', 'GET',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.ListKeyManagementServersApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
            else:
                (data) = self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers', 'GET',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.ListKeyManagementServersApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
                return data
        finally:
            pass

    def update_key_management_server_by_id(self, extId, body, **kwargs):  # noqa: E501
        """Update a key management server

        >>> response = api.update_key_management_server_by_id(extId, body)

        This method makes a synchronous HTTP request by default. To make an asynchronous HTTP request, please pass async_req=True.

        >>> thread = api.update_key_management_server_by_id(extId, body, async_req=True)
        >>> result = thread.get()

        :param extId: Unique identifier for the key management server of type UUID.
        :type extId:
            :class:`str`, required
        :param body: Parameters to update a key management server.
        :type body:
            :class:`~ntnx_security_py_client.models.security.v4.config.KeyManagementServer`, required
        :param kwargs: kwargs for the method.
                       The following can be passed for an asynchronous HTTP call::

                           async_req=True
        :return: An instance of class :class:`~ntnx_security_py_client.models.security.v4.config.UpdateKeyManagementServerApiResponse`.

                 If the method is called asynchronously, returns the request thread.
        """ # noqa: E501
        kwargs['_return_http_data_only'] = True

        params = dict(locals())
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        del params['self']

        # verify the required parameter 'extId' is set
        if ('extId' not in params or params['extId'] is None):
            raise ValueError("Missing the required parameter `extId` when calling `update_key_management_server_by_id`")  # noqa: E501
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `update_key_management_server_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'extId' in params and params['extId'] is not None:
            path_params['extId'] = params['extId']  # noqa: E501

        query_params = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client._select_header_accept(
            ['application/json'])  # noqa: E501
        if 'Accept' in params and params.get('Accept') is not None:
            header_params['Accept'] = params.get('Accept')

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client._select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501
        if 'Content-Type' in params and params.get('Content-Type') is not None:
            header_params['Content-Type'] = params.get('Content-Type')

        # Process operation specific headers
        extra_params = []
        extra_params.append('async_req')
        extra_params.append('_return_http_data_only')
        extra_params.append('_preload_content')
        extra_params.append('_request_timeout')
        all_params = set(['extId', 'body'])
        all_params.update(extra_params)
        for key, val in six.iteritems(params):
            if val is not None and key.lower() not in self.__headers_to_skip and key not in all_params:
                if key.lower() == 'if_match'.lower():
                    key = 'If-Match'
                elif key.lower() == 'if_none_match'.lower():
                    key = 'If-None-Match'
                header_params[key] = val

        form_params = []
        local_var_files = {}

        # Authentication setting
        auth_settings = ['apiKeyAuthScheme', 'basicAuthScheme']  # noqa: E501

        body_params = None
        if 'body' in params and params['body'] is not None:
            body_params = params['body']

        try:
            if kwargs.get('async_req'):
                return self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'PUT',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.UpdateKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
            else:
                (data) = self.api_client._call_api(
                    '/api/security/v4.0/config/key-management-servers/{extId}', 'PUT',
                    path_params,
                    query_params,
                    header_params,
                    body=body_params,
                    post_params=form_params,
                    files=local_var_files,
                    response_type='security.v4.config.UpdateKeyManagementServerApiResponse',  # noqa: E501
                    auth_settings=auth_settings,
                    async_req=params.get('async_req'),
                    _return_http_data_only=params.get('_return_http_data_only'),
                    _preload_content=params.get('_preload_content', True),
                    _request_timeout=params.get('_request_timeout'),
                    collection_formats=collection_formats)
                return data
        finally:
            pass



# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.security.v4.report.SoftwareType import SoftwareType  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.VulnerabilitySummary import VulnerabilitySummary  # noqa: F401,E501

class VulnerabilitySummaryProjection(VulnerabilitySummary):

    """

    :param cluster_ext_id: (:attr:`cluster_ext_id`) UUID of the cluster.
    :type cluster_ext_id: 
    
    
    :param installed_version: (:attr:`installed_version`) AOS version on that cluster
    :type installed_version: 
     max=199 chars
    
    :param cve_ids: (:attr:`cve_ids`) The CVE ids associated with this CESA
    :type cve_ids: 
    
    
    :param software_type: (:attr:`software_type`) 
    :type software_type: 
    
    
    :param ext_id: (:attr:`ext_id`) A globally unique identifier of an instance that is suitable for external consumption. 
    :type ext_id: 
    
    
    :param links: (:attr:`links`) A HATEOAS style link for the response.  Each link contains a user-friendly name identifying the link and an address for retrieving the particular resource. 
    :type links: 
    
    
    :param tenant_id: (:attr:`tenant_id`) A globally unique identifier that represents the tenant that owns this entity. The system automatically assigns it, and it and is immutable from an API consumer perspective (some use cases may cause this ID to change - For instance, a use case may require the transfer of ownership of the entity, but these cases are handled automatically on the server). 
    :type tenant_id: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'cluster_ext_id': 'str',
        'installed_version': 'str',
        'cve_ids': 'list[str]',
        'software_type': 'security.v4.report.SoftwareType',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'cluster_ext_id': 'clusterExtId',
        'installed_version': 'installedVersion',
        'cve_ids': 'cveIds',
        'software_type': 'softwareType',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, cluster_ext_id=None, installed_version=None, cve_ids=None, software_type=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        VulnerabilitySummary.__init__(self, cluster_ext_id, installed_version, cve_ids, software_type, ext_id, links, tenant_id, *args, **kwargs)
        self.discriminator = None

    def _initialize_object_type(self):
        return 'security.v4.report.VulnerabilitySummaryProjection'

    def _initialize_object_version(self):
        return 'v4.r0'


    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(VulnerabilitySummaryProjection, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VulnerabilitySummaryProjection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_security_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.Severity import Severity  # noqa: F401,E501

class Vulnerability(ExternalizableAbstractModel):

    """Contains all relevant information for a vulnerability

    :param cesa_id: (:attr:`cesa_id`) The CESA id
    :type cesa_id: 
     max=199 chars
    
    :param cve_ids: (:attr:`cve_ids`) The CVE ids associated with this CESA
    :type cve_ids: 
    
    
    :param is_critical: (:attr:`is_critical`) If the vulnerability is critical or not
    :type is_critical: 
    
    
    :param description: (:attr:`description`) The short description of the vulnerability
    :type description: 
     max=499 chars
    
    :param fix_versions: (:attr:`fix_versions`) The AOS versions where the vulnerability was fixed
    :type fix_versions: 
    
    
    :param severity: (:attr:`severity`) 
    :type severity: 
    
    
    :param nxvd_version: (:attr:`nxvd_version`) Version of NXVD data used to calculate vulnerabilities
    :type nxvd_version: 
     max=199 chars
    
    :param nxvd_created_time: (:attr:`nxvd_created_time`) Date of creation of NXVD data used to calculate vulnerabilities
    :type nxvd_created_time: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'cesa_id': 'str',
        'cve_ids': 'list[str]',
        'is_critical': 'bool',
        'description': 'str',
        'fix_versions': 'list[str]',
        'severity': 'security.v4.report.Severity',
        'nxvd_version': 'str',
        'nxvd_created_time': 'datetime',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'cesa_id': 'cesaId',
        'cve_ids': 'cveIds',
        'is_critical': 'isCritical',
        'description': 'description',
        'fix_versions': 'fixVersions',
        'severity': 'severity',
        'nxvd_version': 'nxvdVersion',
        'nxvd_created_time': 'nxvdCreatedTime',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, cesa_id=None, cve_ids=None, is_critical=None, description=None, fix_versions=None, severity=None, nxvd_version=None, nxvd_created_time=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__cesa_id = None
        self.__cve_ids = None
        self.__is_critical = None
        self.__description = None
        self.__fix_versions = None
        self.__severity = None
        self.__nxvd_version = None
        self.__nxvd_created_time = None
        self.discriminator = None
        if cesa_id is not None:
            self.__cesa_id = cesa_id
        if cve_ids is not None:
            self.__cve_ids = cve_ids
        if is_critical is not None:
            self.__is_critical = is_critical
        if description is not None:
            self.__description = description
        if fix_versions is not None:
            self.__fix_versions = fix_versions
        if severity is not None:
            self.__severity = severity
        if nxvd_version is not None:
            self.__nxvd_version = nxvd_version
        if nxvd_created_time is not None:
            self.__nxvd_created_time = nxvd_created_time

    def _initialize_object_type(self):
        return 'security.v4.report.Vulnerability'

    def _initialize_object_version(self):
        return 'v4.r0'


    @property
    def cesa_id(self):
        """
        The CESA id

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__cesa_id

    @cesa_id.setter
    def cesa_id(self, cesa_id):
        if cesa_id is not None and len(cesa_id) > 199:
            raise ValueError("Invalid value for `cesa_id`, length must be less than or equal to `199`")  # noqa: E501

        self.__cesa_id = cesa_id

    @property
    def cve_ids(self):
        """
        The CVE ids associated with this CESA

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__cve_ids

    @cve_ids.setter
    def cve_ids(self, cve_ids):

        self.__cve_ids = cve_ids

    @property
    def is_critical(self):
        """
        If the vulnerability is critical or not

        :type:
                :class:`~bool`
        """  # noqa: E501
        return self.__is_critical

    @is_critical.setter
    def is_critical(self, is_critical):

        self.__is_critical = is_critical

    @property
    def description(self):
        """
        The short description of the vulnerability

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__description

    @description.setter
    def description(self, description):
        if description is not None and len(description) > 499:
            raise ValueError("Invalid value for `description`, length must be less than or equal to `499`")  # noqa: E501

        self.__description = description

    @property
    def fix_versions(self):
        """
        The AOS versions where the vulnerability was fixed

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__fix_versions

    @fix_versions.setter
    def fix_versions(self, fix_versions):

        self.__fix_versions = fix_versions

    @property
    def severity(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.security.v4.report.Severity`
        """  # noqa: E501
        return self.__severity

    @severity.setter
    def severity(self, severity):

        self.__severity = severity

    @property
    def nxvd_version(self):
        """
        Version of NXVD data used to calculate vulnerabilities

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__nxvd_version

    @nxvd_version.setter
    def nxvd_version(self, nxvd_version):
        if nxvd_version is not None and len(nxvd_version) > 199:
            raise ValueError("Invalid value for `nxvd_version`, length must be less than or equal to `199`")  # noqa: E501

        self.__nxvd_version = nxvd_version

    @property
    def nxvd_created_time(self):
        """
        Date of creation of NXVD data used to calculate vulnerabilities

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__nxvd_created_time

    @nxvd_created_time.setter
    def nxvd_created_time(self, nxvd_created_time):

        self.__nxvd_created_time = nxvd_created_time

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(Vulnerability, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Vulnerability):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path

class ScheduleType(object):

    """Contains possible values for scheduling a task.


    Allowed enum values:

        - _UNKNOWN
        - _REDACTED
        - HOURLY: Schedule per hour
        - DAILY: Schedule per day
        - WEEKLY: Schedule per week
        - MONTHLY: Schedule per month
    """
    _UNKNOWN = "$UNKNOWN"
    _REDACTED = "$REDACTED"
    HOURLY = "HOURLY"
    DAILY = "DAILY"
    WEEKLY = "WEEKLY"
    MONTHLY = "MONTHLY"


    def __init__(self, *args, **kwargs):  # noqa: E501
        self.discriminator = None
        # populate hidden vars if not empty
        self._populate_hidden_vars(kwargs)

    def _initialize_object_type(self):
        return 'security.v4.config.ScheduleType'

    def _initialize_object_version(self):
        return 'v4.r0'

    def _populate_hidden_vars(self, kwargs):
        if "_reserved" in kwargs and kwargs["_reserved"] is not None:
            self.__dollar_reserved = kwargs["_reserved"]
        elif hasattr(self, "attribute_map") and "_reserved" in self.attribute_map and self.attribute_map["_reserved"] in kwargs and kwargs[self.attribute_map["_reserved"]] is not None:
            self.__dollar_reserved = kwargs[self.attribute_map["_reserved"]]
        else :
            self.__dollar_reserved = {"$fv": self._initialize_object_version()}
        if "_unknown_fields" in kwargs and kwargs["_unknown_fields"] is not None:
            self.__dollar_unknown_fields = kwargs["_unknown_fields"]
        elif hasattr(self, "attribute_map") and "_unknown_fields" in self.attribute_map and self.attribute_map["_unknown_fields"] in kwargs and kwargs[self.attribute_map["_unknown_fields"]] is not None:
            self.__dollar_unknown_fields = kwargs[self.attribute_map["_unknown_fields"]]
        else :
            self.__dollar_unknown_fields = {}
        if "_object_type" in kwargs and kwargs["_object_type"] is not None:
            self.__dollar_object_type = kwargs["_object_type"]
        elif hasattr(self, "attribute_map") and "_object_type" in self.attribute_map and self.attribute_map["_object_type"] in kwargs and kwargs[self.attribute_map["_object_type"]] is not None:
            self.__dollar_object_type = kwargs[self.attribute_map["_object_type"]]
        else:
            self.__dollar_object_type = self._initialize_object_type()

    def get_object_type(self):
        return self.__dollar_object_type

    def get_reserved(self):
        return self.__dollar_reserved

    def get_unknown_fields(self):
        return self.__dollar_unknown_fields

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr in vars(self):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(ScheduleType, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ScheduleType):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_security_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.BucketsAccessKeyStatusType import BucketsAccessKeyStatusType  # noqa: F401,E501
from ntnx_security_py_client.models.iam.v4.authn.CreationType import CreationType  # noqa: F401,E501

class BucketsAccessKey(ExternalizableAbstractModel):

    """Information of bucket access key.

    :param access_key_name: (:attr:`access_key_name`) Name of the bucket access key.
    :type access_key_name: required
     min=1 chars, max=255 chars
    
    :param secret_access_key: (:attr:`secret_access_key`) This represents secret access key, which will be returned only during access key creation.
    :type secret_access_key: 
    
    
    :param user_id: (:attr:`user_id`) User identifier who owns the bucket access key.
    :type user_id: 
    
    
    :param created_time: (:attr:`created_time`) Creation time for the bucket access key.
    :type created_time: 
    
    
    :param last_updated_by: (:attr:`last_updated_by`) Entity that updated the buckets access key.
    :type last_updated_by: 
    
    
    :param assigned_to: (:attr:`assigned_to`) External client to whom this key is allocated.
    :type assigned_to: 
     min=3 chars, max=255 chars
    
    :param creation_type: (:attr:`creation_type`) 
    :type creation_type: 
    
    
    :param expiry_time: (:attr:`expiry_time`) The expiry time of the buckets access Key.
    :type expiry_time: 
    
    
    :param status: (:attr:`status`) 
    :type status: 
    
    
    :param created_by: (:attr:`created_by`) Service account user who created the buckets access key.
    :type created_by: 
    
    
    :param last_updated_time: (:attr:`last_updated_time`) Creation time for the bucket access key.
    :type last_updated_time: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'access_key_name': 'str',
        'secret_access_key': 'str',
        'user_id': 'str',
        'created_time': 'datetime',
        'last_updated_by': 'str',
        'assigned_to': 'str',
        'creation_type': 'iam.v4.authn.CreationType',
        'expiry_time': 'datetime',
        'status': 'iam.v4.authn.BucketsAccessKeyStatusType',
        'created_by': 'str',
        'last_updated_time': 'datetime',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'access_key_name': 'accessKeyName',
        'secret_access_key': 'secretAccessKey',
        'user_id': 'userId',
        'created_time': 'createdTime',
        'last_updated_by': 'lastUpdatedBy',
        'assigned_to': 'assignedTo',
        'creation_type': 'creationType',
        'expiry_time': 'expiryTime',
        'status': 'status',
        'created_by': 'createdBy',
        'last_updated_time': 'lastUpdatedTime',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, access_key_name=None, secret_access_key=None, user_id=None, created_time=None, last_updated_by=None, assigned_to=None, creation_type=None, expiry_time=None, status=None, created_by=None, last_updated_time=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__access_key_name = None
        self.__secret_access_key = None
        self.__user_id = None
        self.__created_time = None
        self.__last_updated_by = None
        self.__assigned_to = None
        self.__creation_type = None
        self.__expiry_time = None
        self.__status = None
        self.__created_by = None
        self.__last_updated_time = None
        self.discriminator = None
        self.__access_key_name = access_key_name
        if secret_access_key is not None:
            self.__secret_access_key = secret_access_key
        if user_id is not None:
            self.__user_id = user_id
        if created_time is not None:
            self.__created_time = created_time
        if last_updated_by is not None:
            self.__last_updated_by = last_updated_by
        if assigned_to is not None:
            self.__assigned_to = assigned_to
        if creation_type is not None:
            self.__creation_type = creation_type
        if expiry_time is not None:
            self.__expiry_time = expiry_time
        if status is not None:
            self.__status = status
        if created_by is not None:
            self.__created_by = created_by
        if last_updated_time is not None:
            self.__last_updated_time = last_updated_time

    def _initialize_object_type(self):
        return 'iam.v4.authn.BucketsAccessKey'

    def _initialize_object_version(self):
        return 'v4.r1.b1'


    @property
    def access_key_name(self):
        """
        Name of the bucket access key.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__access_key_name

    @access_key_name.setter
    def access_key_name(self, access_key_name):
        if access_key_name is None:
            raise ValueError("Invalid value for `access_key_name`, must not be `None`")  # noqa: E501
        if access_key_name is not None and len(access_key_name) > 255:
            raise ValueError("Invalid value for `access_key_name`, length must be less than or equal to `255`")  # noqa: E501
        if access_key_name is not None and len(access_key_name) < 1:
            raise ValueError("Invalid value for `access_key_name`, length must be greater than or equal to `1`")  # noqa: E501
        if access_key_name is not None and not re.search('^[^<>;\'()&+%\/\\\\\"`]*$', access_key_name):  # noqa: E501
            raise ValueError(r"Invalid value for `access_key_name`, must be a follow pattern or equal to `/^[^<>;'()&+%\/\\\\\"`]*$/`")  # noqa: E501

        self.__access_key_name = access_key_name

    @property
    def secret_access_key(self):
        """
        This represents secret access key, which will be returned only during access key creation.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__secret_access_key

    @secret_access_key.setter
    def secret_access_key(self, secret_access_key):

        self.__secret_access_key = secret_access_key

    @property
    def user_id(self):
        """
        User identifier who owns the bucket access key.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__user_id

    @user_id.setter
    def user_id(self, user_id):
        if user_id is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', user_id):  # noqa: E501
            raise ValueError(r"Invalid value for `user_id`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__user_id = user_id

    @property
    def created_time(self):
        """
        Creation time for the bucket access key.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__created_time

    @created_time.setter
    def created_time(self, created_time):

        self.__created_time = created_time

    @property
    def last_updated_by(self):
        """
        Entity that updated the buckets access key.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        if last_updated_by is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', last_updated_by):  # noqa: E501
            raise ValueError(r"Invalid value for `last_updated_by`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__last_updated_by = last_updated_by

    @property
    def assigned_to(self):
        """
        External client to whom this key is allocated.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__assigned_to

    @assigned_to.setter
    def assigned_to(self, assigned_to):
        if assigned_to is not None and len(assigned_to) > 255:
            raise ValueError("Invalid value for `assigned_to`, length must be less than or equal to `255`")  # noqa: E501
        if assigned_to is not None and len(assigned_to) < 3:
            raise ValueError("Invalid value for `assigned_to`, length must be greater than or equal to `3`")  # noqa: E501

        self.__assigned_to = assigned_to

    @property
    def creation_type(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.iam.v4.authn.CreationType`
        """  # noqa: E501
        return self.__creation_type

    @creation_type.setter
    def creation_type(self, creation_type):

        self.__creation_type = creation_type

    @property
    def expiry_time(self):
        """
        The expiry time of the buckets access Key.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__expiry_time

    @expiry_time.setter
    def expiry_time(self, expiry_time):

        self.__expiry_time = expiry_time

    @property
    def status(self):
        """
        

        :type:
            :class:`~ntnx_security_py_client.models.iam.v4.authn.BucketsAccessKeyStatusType`
        """  # noqa: E501
        return self.__status

    @status.setter
    def status(self, status):

        self.__status = status

    @property
    def created_by(self):
        """
        Service account user who created the buckets access key.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__created_by

    @created_by.setter
    def created_by(self, created_by):
        if created_by is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', created_by):  # noqa: E501
            raise ValueError(r"Invalid value for `created_by`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__created_by = created_by

    @property
    def last_updated_time(self):
        """
        Creation time for the bucket access key.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__last_updated_time

    @last_updated_time.setter
    def last_updated_time(self, last_updated_time):

        self.__last_updated_time = last_updated_time

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(BucketsAccessKey, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BucketsAccessKey):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
from __future__ import absolute_import

import os
import copy
import logging
from logging.handlers import TimedRotatingFileHandler
import multiprocessing
import sys
import time
import urllib3

import six
from six.moves import http_client as httplib


class TypeWithDefault(type):
    def __init__(cls, name, bases, dct):
        super(TypeWithDefault, cls).__init__(name, bases, dct)
        cls._default = None

    def __call__(cls):
        if cls._default is None:
            cls._default = type.__call__(cls)
        return copy.copy(cls._default)

    def set_default(cls, default):
        cls._default = copy.copy(default)


class Configuration(six.with_metaclass(TypeWithDefault, object)):
    """A class for the configuration of :class:`~ntnx_security_py_client.api_client.ApiClient`.

    This class contains certain configuration methods and data required by the api client.

    :param scheme: (:attr:`scheme`) URI scheme for connecting to the cluster (HTTP or HTTPS using SSL/TLS) (**Default** https)
    :type scheme: :class:`str`
    :param host: (:attr:`host`) A host name to connect which can be either IPv4, IPv6 or FQDN (**Default** localhost)
    :type host: :class:`str`, required
    :param port: (:attr:`port`) A port number to connect (**Default** 9440)
    :type port: :class:`int`
    :param username: (:attr:`username`) A username for HTTP basic authentication
    :type username: :class:`str`, required
    :param password: (:attr:`password`) A password for HTTP basic authentication
    :type password: :class:`str`, required
    :param debug: (:attr:`debug`) Runs the client in debug mode by enabling/disabling debug logging (**Default** False)
    :type debug: :class:`bool`
    :param verify_ssl: (:attr:`verify_ssl`) Verify SSL certificate of cluster the client will connect to (**Default** True)
    :type verify_ssl: :class:`bool`
    :param max_redirects: (:attr:`max_redirects`) Maximum redirect attempts to be made (**Default** 1)
    :type max_redirects: :class:`int`
    :param max_retry_attempts: (:attr:`max_retry_attempts`) Maximum retry attempts to be made in case of status codes [408, 429, 503, 504] (**Default** 5)
    :type max_retry_attempts: :class:`int`
    :param backoff_factor: (:attr:`backoff_factor`) Backoff factor by which the retry request is delayed with specific number of seconds (**Default** 3).

        This is calculated by the following formula: ::

        {backoff_factor} * (2 * ({number of retries so far} - 1))
    :type backoff_factor: :class:`float`
    :param logger_file: (:attr:`logger_file`) File location to which logs are written to
    :type logger_file: :class:`str`
    :param connect_timeout: (:attr:`connect_timeout`) Connection timeout in milliseconds for all operations (**Default** 30000)
    :type connect_timeout: :class:`int`
    :param read_timeout: (:attr:`read_timeout`) Read timeout in milliseconds for all operations (**Default** 30000)
    :type read_timeout: :class:`int`
    :param download_directory: (:attr:`download_directory`) Directory location on local for files to download
    :type download_directory: :class:`str`
    :param download_chunk_size: (:attr:`download_chunk_size`) Chunk size in bytes for files to download (**Default** 8*1024 bytes)
    :type download_chunk_size: :class:`int`
    """  # noqa: E501

    def __init__(self):

        """Constructor"""
        self.__scheme = "https"

        # Default host name
        self.__host = "localhost"

        # Default port number
        self.__port = 9440

        # Maximum number of allowed redirects for a HTTP call
        self.__max_redirects = 1

        # Maximum number of allowed retries for a HTTP call
        self.__max_retry_attempts = 5

        # Backoff factor by which the retry request is delayed with specific number of seconds.
        self.__backoff_factor = 3

        self.__user_agent = 'Nutanix-ntnx_security_py_client/4.0.1'

        # Directory path for downloading files
        self.__download_directory = os.path.abspath(os.getcwd())
        # Chunk size for downloading files
        self.__download_chunk_size = 8*1024

        # Authentication Settings
        # HTTP Basic Auth
        self.__username = None
        self.__password = None

        # API Key authentication (NOTE: SDK currently supports basic auth only)
        # Dictionary to store API key(s)
        self.api_key = {}
        # Dictionary to store API prefix (e.g. Bearer)
        self.api_key_prefix = {}

        # Logging Settings
        self.logger = {}
        self.logger["package_logger"] = logging.getLogger("ntnx_security_py_client")

        # Log format
        self.logger_format = logging.Formatter('%(asctime)sZ %(levelname)s [%(threadName)s:%(name)s:%(lineno)d] %(message)s')
        self.logger_format.converter = time.gmtime

        # Log stream handler
        self.logger_stream_handler = None
        # Log file handler
        self.logger_file_handler = None
        # Debug file location
        self.logger_file = None

        # Debug switch
        self.__debug = False

        # SSL/TLS verification
        # Set this to false to skip verifying SSL certificate received from the https server when calling API
        self.__verify_ssl = True

        # Set this to customize the certificate file to verify the peer.
        self.root_ca_certificate_file = None
        # client certificate file
        self.client_certificate_file = None
        # client key file
        self.client_key_file = None
        # Set this to True/False to enable/disable SSL hostname verification.
        self.assert_hostname = None

        # urllib3 connection pool's maximum number of connections saved
        # per pool. urllib3 uses 1 connection as default value, but this is
        # not the best value when you are making a lot of possibly parallel
        # requests to the same host, which is often the case here.
        # cpu_count * 5 is used as default value to increase performance.
        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5

        # Proxy Parameters
        self.__proxy_scheme = None
        self.__proxy_host = None
        self.__proxy_port = None
        self.__proxy_username = None
        self.__proxy_password = None

        # Safe chars for path_param
        self.safe_chars_for_path_param = ''

        # Request timeout in milliseconds
        self.__default_connect_timeout = 30000
        self.__default_read_timeout = 30000
        self.__connect_timeout = None
        self.__read_timeout = None

    @property
    def scheme(self):
        """URI scheme for connecting to the cluster (HTTP or HTTPS using SSL/TLS) (**Default** https).

        :type: :class:`str`
        """
        return self.__scheme

    @scheme.setter
    def scheme(self, value):
        self.__scheme = value

    @property
    def host(self):
        """Host name to connect which can be either IPv4, IPv6 or FQDN (**Default** localhost).

        :type: :class:`str`
        """
        return self.__host

    @host.setter
    def host(self, value):
        self.__host = value

    @property
    def port(self):
        """Port number to connect (**Default** 9440).

        :type: :class:`int`
        """
        return self.__port

    @port.setter
    def port(self, value):
        self.__port = value

    @property
    def download_directory(self):
        """Directory path for downloading files (**Default** current directory).

        :type: :class:`str`
        """
        return self.__download_directory

    @download_directory.setter
    def download_directory(self, value):
        self.__download_directory = value

    @property
    def download_chunk_size(self):
        """Chunk size for downloading files (**Default** 8*1024 bytes).

        :type: :class:`int`
        """
        return self.__download_chunk_size

    @download_chunk_size.setter
    def download_chunk_size(self, value):
        self.__download_chunk_size = value

    @property
    def max_redirects(self):
        """Maximum allowed redirect attempts for a HTTP call (**Default** 1).

        :type: :class:`int`
        """
        return self.__max_redirects

    @max_redirects.setter
    def max_redirects(self, value):
        self.__max_redirects = value

    @property
    def max_retry_attempts(self):
        """Maximum allowed retry attempts for a HTTP call in case of response status codes [408, 429, 503, 504] (**Default** 5).

        :type: :class:`int`
        """
        return self.__max_retry_attempts

    @max_retry_attempts.setter
    def max_retry_attempts(self, value):
        self.__max_retry_attempts = value

    @property
    def backoff_factor(self):
        """Backoff factor by which the retry request is delayed with specific number of seconds (**Default** 3).

        This is calculated by the following formula: ::

            {backoff_factor} * (2 * ({number of retries so far} - 1))

        :type: :class:`float`
        """
        return self.__backoff_factor

    @backoff_factor.setter
    def backoff_factor(self, value):
        self.__backoff_factor = value

    @property
    def user_agent(self):
        """Value for User-Agent header.

        :type: :class:`str`
        """
        return self.__user_agent

    @property
    def username(self):
        """Username to connect to a cluster.

        :type: :class:`str`
        """
        return self.__username

    @username.setter
    def username(self, value):
        self.__username = value

    @property
    def password(self):
        """Password to connect to a cluster.

        :type: :class:`str`
        """
        return self.__password

    @password.setter
    def password(self, value):
        self.__password = value

    @property
    def verify_ssl(self):
        """A flag to enable/disable verification the SSL certificate of target cluster.

        :type: :class:`bool`
        """
        return self.__verify_ssl

    @verify_ssl.setter
    def verify_ssl(self, value):
        self.__verify_ssl = value

    @property
    def proxy_scheme(self):
        """Scheme for the proxy URI.

        :type: :class:`str`
        """
        return self.__proxy_scheme

    @proxy_scheme.setter
    def proxy_scheme(self, value):
        self.__proxy_scheme = value

    @property
    def proxy_host(self):
        """Host for the proxy URI.

        :type: :class:`str`
        """
        return self.__proxy_host

    @proxy_host.setter
    def proxy_host(self, value):
        self.__proxy_host = value

    @property
    def proxy_port(self):
        """Port number for the proxy URI.

        :type: :class:`int`
        """
        return self.__proxy_port

    @proxy_port.setter
    def proxy_port(self, value):
        self.__proxy_port = value

    @property
    def proxy_username(self):
        """Username for the proxy authentication.

        :type: :class:`str`
        """
        return self.__proxy_username

    @proxy_username.setter
    def proxy_username(self, value):
        self.__proxy_username = value

    @property
    def proxy_password(self):
        """Password for the proxy authentication.

        :type: :class:`str`
        """
        return self.__proxy_password

    @proxy_password.setter
    def proxy_password(self, value):
        self.__proxy_password = value

    @property
    def logger_file(self):
        """File location to which debug logs are written to.

        If the logger_file is None, then a  logger is configured with a console stream handler and all the file
        handlers are removed.
        Otherwise, a logger is configured with a file handler and stream handlers are removed.

        Configured file handlers are time based handlers which are rotated everyday at midnight.

        :type: :class:`str`
        """
        return self.__logger_file

    @logger_file.setter
    def logger_file(self, value):
        self.__logger_file = value
        if self.__logger_file:
            # If set logging file,
            # then add file handler and remove stream handler.
            self.logger_file_handler = logging.handlers.TimedRotatingFileHandler(self.__logger_file, 'midnight', 1)
            self.logger_file_handler.suffix = "%Y-%m-%d"
            self.logger_file_handler.setFormatter(self.logger_format)
            for _, logger in six.iteritems(self.logger):
                logger.addHandler(self.logger_file_handler)
                if self.logger_stream_handler:
                    logger.removeHandler(self.logger_stream_handler)
        else:
            # If not set logging file,
            # then add stream handler and remove file handler.
            self.logger_stream_handler = logging.StreamHandler()
            self.logger_stream_handler.setFormatter(self.logger_format)
            for _, logger in six.iteritems(self.logger):
                logger.addHandler(self.logger_stream_handler)
                if self.logger_file_handler:
                    logger.removeHandler(self.logger_file_handler)

    @property
    def debug(self):
        """Runs the client in debug mode by enabling/disabling debug logging (**Default** False).

        Default logging level is INFO and all associated loggers' levels are toggled between INFO and DEBUG by this flag.

        :type: :class:`bool`
        """
        return self.__debug

    @debug.setter
    def debug(self, value):
        self.__debug = value
        if self.__debug:
            # if debug status is True, turn on debug logging
            for _, logger in six.iteritems(self.logger):
                logger.setLevel(logging.DEBUG)
        else:
            # if debug status is False, turn off debug logging,
            # setting log level to default `logging.INFO`
            for _, logger in six.iteritems(self.logger):
                logger.setLevel(logging.INFO)

    @property
    def logger_format(self):
        """The log format for file or stream log handler.

        :type: :class:`str`
        """
        return self.__logger_format

    @logger_format.setter
    def logger_format(self, value):
        self.__logger_format = value

    @property
    def default_connect_timeout(self):
        """Default connection timeout in milliseconds for a HTTP request (**Default** 30000).

        :type: :class:`int`
        """

        return self.__default_connect_timeout

    @property
    def default_read_timeout(self):
        """Default read timeout in milliseconds for a HTTP request (**Default** 30000).

        :type: :class:`int`
        """

        return self.__default_read_timeout

    @property
    def connect_timeout(self):
        """Connect timeout for an operation in milliseconds.

        :type: :class:`int`
        """

        return self.__connect_timeout

    @connect_timeout.setter
    def connect_timeout(self, value):
        self.__connect_timeout = value

    @property
    def read_timeout(self):
        """Read timeout for an operation in milliseconds.

        :type: :class:`int`
        """
        return self.__read_timeout

    @read_timeout.setter
    def read_timeout(self, value):
        self.__read_timeout = value

    def set_api_key(self, key):
        """Set API key

        :param key: API key for authentication
        :type key: :class:`str`
        """
        if key is None or type(key) is str:
            self.api_key['X-ntnx-api-key'] = key
        else:
            self.api_key['X-ntnx-api-key'] = None

    def _get_api_key_with_prefix(self, identifier):
        """Gets API key (with prefix if set).

        :param identifier: The identifier of apiKey
        :type identifier: :class:`str`
        ...
        :return: A token for api key authentication
        :rtype: :class:`str`
        """
        if (self.api_key.get(identifier) and
                self.api_key_prefix.get(identifier)):
            return self.api_key_prefix[identifier] + ' ' + self.api_key[identifier]  # noqa: E501
        elif self.api_key.get(identifier):
            return self.api_key[identifier]

    def get_basic_auth_token(self):
        """Gets HTTP basic authentication header (string).

        :return: A token for basic HTTP authentication.
        :rtype: :class:`str`
        """
        if self.__username and self.__password:
            return urllib3.util.make_headers(
                basic_auth=self.__username + ':' + self.__password
            ).get('authorization')

    def _auth_settings(self):
        """Gets Auth Settings configuration.
        """
        return {
            'apiKeyAuthScheme':
                {
                    'type': 'api_key',
                    'in': 'header',
                    'key': 'X-ntnx-api-key',
                    'value': self._get_api_key_with_prefix('X-ntnx-api-key')
                },
            'basicAuthScheme':
                {
                    'type': 'basic',
                    'in': 'header',
                    'key': 'Authorization',
                    'value': self.get_basic_auth_token()
                },
        }

    def to_debug_report(self):
        """Prints the information about current OS, Python, API and SDK versions for debugging purposes
        """
        return "Python SDK Debug Report:\n"\
               "OS: {env}\n"\
               "Python Version: {pyversion}\n"\
               "Version of the API: 4.0.1\n"\
               "SDK Package Version: 4.0.1".\
               format(env=sys.platform, pyversion=sys.version)


# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import urllib3
from urllib3.connectionpool import HTTPConnectionPool
from urllib3.connectionpool import HTTPSConnectionPool

class CustomPoolManager(urllib3.PoolManager):
    def __init__(self, *args, **kwargs):
        super().__init__(
            num_pools=args[0],
            maxsize=args[1],
            cert_reqs=args[2],
            ca_certs=args[3],
            cert_file=args[4],
            key_file=args[5],
            retries = args[6],
            **kwargs
        )

        self.pool_classes_by_scheme = {
            'http': HTTPConnectionPool,
            'https': HTTPSConnectionPool,
        }

    # Override urlopen to make default value of the redirect parameter as False
    def urlopen(self, method, url, redirect=False, **kw):
        return super().urlopen(method, url, redirect, **kw)




# coding: utf-8

"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
from __future__ import absolute_import

import datetime
import atexit
import json
import mimetypes
from multiprocessing.pool import ThreadPool
import os
import re
import tempfile
import sys
import uuid
import logging

# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import quote

from pathlib import Path


try:
    import urllib3
except ImportError:
    raise ImportError('Api client requires urllib3.')

from ntnx_security_py_client.configuration import Configuration
import ntnx_security_py_client.models
from ntnx_security_py_client import rest
from ntnx_security_py_client.models.common.v1.response.ApiResponseMetadata import ApiResponseMetadata
from ntnx_security_py_client.models.common.v1.config.Flag import Flag

PY2 = sys.version_info[0] < 3

logger = logging.getLogger(__name__)

class ApiClient(object):

    """API client to handle the client-server communication, and is invariant across implementations.

    :param configuration: Configuration object for this client
    :type configuration: :class:`~ntnx_vmm_py_client.configuration.Configuration`, required
    """  # noqa: E501

    SDK_VERSION = "v4.0"
    PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
    NATIVE_TYPES_MAPPING = {
        'int': int,
        'long': int if six.PY3 else long,  # noqa: F821
        'float': float,
        'str': str,
        'bool': bool,
        'date': datetime.date,
        'datetime': datetime.datetime,
        'object': object,
    }
    DUPLICATE_SCHEMA_MAPPING = {
    # Populated if multiple schemas exist with the same name
        'security.v4.management.AssociatePoliciesSpec' : 'AssociatePoliciesSpec',
        'common.v1.config.IPv4Address' : 'IPv4Address',
        'common.v1.config.FQDN' : 'FQDN',
        'security.v4.management.SecuredEntityType' : 'SecuredEntityType',
        'common.v1.config.IPv6Address' : 'IPv6Address',
        'OneOfsecurity.v4.config.DeleteCredentialApiResponsedata' : 'DeleteCredentialApiResponsedata',
        'security.v4.config.KeyManagementServer' : 'KeyManagementServer',
        'security.v4.config.PasswordVerificationStatus' : 'PasswordVerificationStatus',
        'security.v4.report.SummaryProjection' : 'SummaryProjection',
        'security.v4.config.BmcCredential' : 'BmcCredential',
        'common.v1.config.Flag' : 'Flag',
        'security.v4.config.UpdateKeyManagementServerApiResponse' : 'UpdateKeyManagementServerApiResponse',
        'security.v4.error.SchemaValidationError' : 'SchemaValidationError',
        'OneOfsecurity.v4.config.ListKeyManagementServersApiResponsedata' : 'ListKeyManagementServersApiResponsedata',
        'security.v4.config.VcenterCredential' : 'VcenterCredential',
        'OneOfsecurity.v4.config.UpdateKeyManagementServerApiResponsedata' : 'UpdateKeyManagementServerApiResponsedata',
        'security.v4.report.StigProjection' : 'StigProjection',
        'prism.v4.config.TaskReference' : 'TaskReference',
        'security.v4.management.ApprovalPolicy' : 'ApprovalPolicy',
        'security.v4.management.PolicyType' : 'PolicyType',
        'iam.v4.authn.BucketsAccessKey' : 'BucketsAccessKey',
        'security.v4.management.EventType' : 'EventType',
        'security.v4.common.Cluster' : 'Cluster',
        'security.v4.config.UpdateCredentialApiResponse' : 'UpdateCredentialApiResponse',
        'iam.v4.authn.CreationType' : 'CreationType',
        'security.v4.report.IssueSummary' : 'IssueSummary',
        'security.v4.config.VerifyPassword' : 'VerifyPassword',
        'OneOfsecurity.v4.config.DeleteKeyManagementServerApiResponsedata' : 'DeleteKeyManagementServerApiResponsedata',
        'security.v4.config.DeleteCredentialApiResponse' : 'DeleteCredentialApiResponse',
        'security.v4.report.Stig' : 'Stig',
        'iam.v4.authn.UserType' : 'UserType',
        'security.v4.common.Timescale' : 'Timescale',
        'security.v4.report.Severity' : 'Severity',
        'common.v1.config.MapOfStringWrapper' : 'MapOfStringWrapper',
        'common.v1.config.IPAddressOrFQDN' : 'IPAddressOrFQDN',
        'security.v4.management.AssociatePoliciesApiResponse' : 'AssociatePoliciesApiResponse',
        'security.v4.error.SchemaValidationErrorMessage' : 'SchemaValidationErrorMessage',
        'security.v4.report.VulnerabilitySummaryProjection' : 'VulnerabilitySummaryProjection',
        'common.v1.response.ApiResponseMetadata' : 'ApiResponseMetadata',
        'security.v4.config.IntersightConnectionType' : 'IntersightConnectionType',
        'security.v4.management.UpdateApprovalPolicyApiResponse' : 'UpdateApprovalPolicyApiResponse',
        'iam.v4.authn.User' : 'User',
        'OneOfsecurity.v4.management.CreateApprovalPolicyApiResponsedata' : 'CreateApprovalPolicyApiResponsedata',
        'security.v4.config.SecurityConfigVisibilitySetting' : 'SecurityConfigVisibilitySetting',
        'security.v4.management.ApproverGroup' : 'ApproverGroup',
        'security.v4.report.ListStigsApiResponse' : 'ListStigsApiResponse',
        'OneOfsecurity.v4.management.AssociatePoliciesApiResponsedata' : 'AssociatePoliciesApiResponsedata',
        'common.v1.config.BasicAuth' : 'BasicAuth',
        'common.v1.response.ExternalizableAbstractModel' : 'ExternalizableAbstractModel',
        'security.v4.config.PasswordStatus' : 'PasswordStatus',
        'OneOfsecurity.v4.config.GetKeyManagementServerApiResponsedata' : 'GetKeyManagementServerApiResponsedata',
        'security.v4.config.ExternalSshAccessConfig' : 'ExternalSshAccessConfig',
        'security.v4.config.KeyBasedAuthCredential' : 'KeyBasedAuthCredential',
        'security.v4.config.ScheduleType' : 'ScheduleType',
        'security.v4.report.ListStigSummariesApiResponse' : 'ListStigSummariesApiResponse',
        'common.v1.config.Message' : 'Message',
        'OneOfsecurity.v4.error.ErrorResponseerror' : 'ErrorResponseerror',
        'security.v4.management.GetApprovalPolicyApiResponse' : 'GetApprovalPolicyApiResponse',
        'security.v4.management.ListApprovalPoliciesApiResponse' : 'ListApprovalPoliciesApiResponse',
        'OneOfsecurity.v4.management.UpdateApprovalPolicyApiResponsedata' : 'UpdateApprovalPolicyApiResponsedata',
        'security.v4.config.ListCredentialsApiResponse' : 'ListCredentialsApiResponse',
        'security.v4.report.SoftwareType' : 'SoftwareType',
        'security.v4.config.CreateKeyManagementServerApiResponse' : 'CreateKeyManagementServerApiResponse',
        'security.v4.management.CreateApprovalPolicyApiResponse' : 'CreateApprovalPolicyApiResponse',
        'OneOfcommon.v1.config.KVPairvalue' : 'KVPairvalue',
        'OneOfsecurity.v4.config.UpdateCredentialApiResponsedata' : 'UpdateCredentialApiResponsedata',
        'security.v4.config.IntersightCredential' : 'IntersightCredential',
        'OneOfsecurity.v4.config.Credentialcredential_details' : 'Credentialcredential_details',
        'security.v4.report.StigSummaryProjection' : 'StigSummaryProjection',
        'security.v4.error.ErrorResponse' : 'ErrorResponse',
        'security.v4.config.BasicAuthCredential' : 'BasicAuthCredential',
        'security.v4.config.DeleteKeyManagementServerApiResponse' : 'DeleteKeyManagementServerApiResponse',
        'security.v4.config.SecurityConfigProjection' : 'SecurityConfigProjection',
        'security.v4.config.Credential' : 'Credential',
        'security.v4.config.ListKeyManagementServersApiResponse' : 'ListKeyManagementServersApiResponse',
        'security.v4.report.StigSummary' : 'StigSummary',
        'security.v4.report.TrendValue' : 'TrendValue',
        'security.v4.report.Summary' : 'Summary',
        'common.v1.config.MessageSeverity' : 'MessageSeverity',
        'security.v4.config.GetKeyManagementServerApiResponse' : 'GetKeyManagementServerApiResponse',
        'security.v4.config.KeyBasedAuth' : 'KeyBasedAuth',
        'security.v4.common.ClusterProjection' : 'ClusterProjection',
        'common.v1.response.ApiLink' : 'ApiLink',
        'security.v4.config.GetCredentialApiResponse' : 'GetCredentialApiResponse',
        'security.v4.report.VulnerabilitySummary' : 'VulnerabilitySummary',
        'security.v4.report.StigStatus' : 'StigStatus',
        'OneOfsecurity.v4.management.ListApprovalPoliciesApiResponsedata' : 'ListApprovalPoliciesApiResponsedata',
        'security.v4.config.CreateCredentialApiResponse' : 'CreateCredentialApiResponse',
        'OneOfsecurity.v4.config.CreateKeyManagementServerApiResponsedata' : 'CreateKeyManagementServerApiResponsedata',
        'security.v4.config.AzureAccessInformation' : 'AzureAccessInformation',
        'security.v4.error.AppMessage' : 'AppMessage',
        'security.v4.report.Vulnerability' : 'Vulnerability',
        'OneOfsecurity.v4.management.GetApprovalPolicyApiResponsedata' : 'GetApprovalPolicyApiResponsedata',
        'security.v4.config.SecurityConfig' : 'SecurityConfig',
        'OneOfsecurity.v4.config.GetCredentialApiResponsedata' : 'GetCredentialApiResponsedata',
        'OneOfsecurity.v4.report.ListStigsApiResponsedata' : 'ListStigsApiResponsedata',
        'security.v4.management.SecuredPolicy' : 'SecuredPolicy',
        'OneOfsecurity.v4.config.CreateCredentialApiResponsedata' : 'CreateCredentialApiResponsedata',
        'common.v1.config.TenantAwareModel' : 'TenantAwareModel',
        'security.v4.management.SecurityValidationSpec' : 'SecurityValidationSpec',
        'OneOfsecurity.v4.config.ListCredentialsApiResponsedata' : 'ListCredentialsApiResponsedata',
        'iam.v4.authn.UserStatusType' : 'UserStatusType',
        'OneOfsecurity.v4.report.ListStigSummariesApiResponsedata' : 'ListStigSummariesApiResponsedata',
        'security.v4.config.VerifyPasswordSpec' : 'VerifyPasswordSpec',
        'security.v4.report.VulnerabilityProjection' : 'VulnerabilityProjection',
        'common.v1.config.KVPair' : 'KVPair',
        'iam.v4.authn.BucketsAccessKeyStatusType' : 'BucketsAccessKeyStatusType'
    }

    def __init__(self, configuration=None):
        if configuration is None:
            configuration = Configuration()
        self.configuration = configuration

        self.__pool = None

        self.rest_client = rest.RESTClientObject(configuration, max_retry_attempts=configuration.max_retry_attempts, backoff_factor=configuration.backoff_factor)
        self.__default_headers = {}

        self.__cookie = None
        self.__refresh_cookie = True


    def __close(self):
        if self.__pool:
            self.__pool.close()
            self.__pool.join()
            self.__pool = None
            if hasattr(atexit, 'unregister'):
                atexit.unregister(self.__close)

            logger.info("Thread pool closed")

    def __initialize_threadpool(self):
        if not self.__pool:
            logger.info("Initializing the thread pool")
            self.__pool = ThreadPool()
            atexit.register(self.__close)

    def add_default_header(self, header_name, header_value):
        """Add a default header to be included in each HTTP request.

        :param header_name: Name of a default header
        :type header_name: :class:`str`
        :param header_value: Value of the provided default header
        :type header_value: :class:`str`
        """
        if header_name == 'Authorization':
            self.__cookie = None

        self.__default_headers[header_name] = header_value

    def __get_request_timeout(self, request_timeout):
        """Gets the Request timeout in seconds.

        :param request_timeout: A list of integers for the [connect, read] timeouts in milliseconds for an operation
        :type request_timeout: :class:`list`
        :return: A final list of integers for the [connect, read] timeouts in milliseconds for an operation
        :rtype: :class:`list`
        """

        if not request_timeout or not isinstance(request_timeout, list) or len(request_timeout) != 2:
            request_timeout = []
            request_timeout.append(self.__get_valid_timeout(self.configuration.connect_timeout, self.configuration.default_connect_timeout))
            request_timeout.append(self.__get_valid_timeout(self.configuration.read_timeout, self.configuration.default_read_timeout))
            return request_timeout
        else:
            # Handle connect timeout
            connect_configuration_timeout_in_seconds = self.__get_valid_timeout(self.configuration.connect_timeout, self.configuration.default_connect_timeout)
            request_timeout[0] = self.__get_valid_timeout(request_timeout[0], connect_configuration_timeout_in_seconds * 1000)
            # Handle read timeout
            read_configuration_timeout_in_seconds = self.__get_valid_timeout(self.configuration.read_timeout, self.configuration.default_read_timeout)
            request_timeout[1] = self.__get_valid_timeout(request_timeout[1], read_configuration_timeout_in_seconds * 1000)
            return request_timeout

    def __get_valid_timeout(self, timeout, default_timeout):
        if not timeout or not isinstance(timeout, (int, float) if six.PY3 else (int, long, float)):  # noqa: E501,F821
            return default_timeout / 1000

        max_timeout_in_milliseconds = 180 * 60 * 1000  # 180 minutes or 3 hours
        if timeout <= 0:
            timeout = default_timeout
        elif timeout > max_timeout_in_milliseconds:
            timeout = max_timeout_in_milliseconds

        return timeout / 1000


    def __update_cookies(self, response_data):
        # Set Cookie information to reuse in subsequent requests for a valid response
        cookies = response_data.getheader('Set-Cookie')
        if cookies is not None:
            cookie = ''
            cookies_list = cookies.split(',')
            for i in range(0, len(cookies_list)):
                final_cookie = cookies_list[i].split(';', 1)[0]
                final_cookie = final_cookie.strip() if '=' in final_cookie else ''
                if final_cookie:
                    cookie += final_cookie + ';'

            # Remove trailing ";"
            if cookie:
                cookie = cookie[:-1]

            self.__cookie = cookie
            self.__refresh_cookie = False
            logger.debug("Retained cookie: %s", cookie)

    def __call_api(
            self, resource_path, method, path_params=None,
            query_params=None, header_params=None, body=None, post_params=None,
            files=None, response_type=None, auth_settings=None,
            _return_http_data_only=None, collection_formats=None,
            _preload_content=True, _request_timeout=None):

        config = self.configuration

        # Add User-Agent header from configuration
        if config.user_agent:
            self.add_default_header('User-Agent', config.user_agent)

        # header parameters
        header_params = header_params or {}
        header_params.update(self.__default_headers)

        # requestID for idempotence functionality
        if 'NTNX-Request-Id' not in header_params :
            requestId = str(uuid.uuid4())
            header_params['NTNX-Request-Id'] = requestId

        if body and hasattr(body, "get_reserved") and 'ETag' in body.get_reserved():
            header_params['If-Match'] = body.get_reserved()['ETag']

        if self.__cookie:
            header_params['Cookie'] = self.__cookie
            if "Authorization" in header_params:
                header_params.pop("Authorization")
        else :
            self.__update_params_for_auth(header_params, query_params, auth_settings)

        if header_params:
            header_params = self.__sanitize_for_serialization(header_params)
            header_params = dict(self.__parameters_to_tuples(header_params, collection_formats))

        # path parameters
        if path_params:
            path_params = self.__sanitize_for_serialization(path_params)
            path_params = self.__parameters_to_tuples(path_params,
                                                    collection_formats)
            for k, v in path_params:
                # specified safe chars, encode everything
                resource_path = resource_path.replace(
                    '{%s}' % k,
                    quote(str(v), safe=config.safe_chars_for_path_param)
                )

        # query parameters
        if query_params:
            query_params = self.__sanitize_for_serialization(query_params)
            query_params = self.__parameters_to_tuples(query_params, collection_formats)

        # post parameters
        if post_params or files:
            post_params = self.__prepare_post_parameters(post_params, files)
            post_params = self.__sanitize_for_serialization(post_params)
            post_params = self.__parameters_to_tuples(post_params, collection_formats)

        # body
        if body and 'application/octet-stream' not in header_params['Content-Type']:
            body = self.__sanitize_for_serialization(body)

        # request url
        url = config.scheme + '://' + config.host + ':' + str(config.port) + resource_path

        # perform request and return response
        response_data = self.request(
            method, url, query_params=query_params, headers=header_params,
            post_params=post_params, body=body,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout)

        # Retry one more time for 401 response with basic auth header and no cookie
        if response_data.status == 401:
            logger.debug("Retrying for an unauthorized request")
            self.__refresh_cookie = True
            if 'Cookie' in header_params:
                header_params.update(self.__default_headers)
                header_params.pop('Cookie')
                self.__update_params_for_auth(header_params, query_params, auth_settings)

            response_data = self.request(
                method, url, query_params=query_params, headers=header_params,
                post_params=post_params, body=body,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout)

            # Raise error if 401 persists
            if response_data.status == 401:
                raise ntnx_security_py_client.rest.ApiException(http_resp=response_data)

        if self.__refresh_cookie:
            self.__update_cookies(response_data)

        self.last_response = response_data
        return_data = response_data

        # deserialize response data
        try:
            if response_data.status != 204:
                # Download a file
                if response_data.getheader(name='Content-Type') in ['application/octet-stream', 'application/pdf', 'application/zip']:
                    download_path = self.__deserialize_file(response_data)
                    download_path = Path(download_path)

                    # convert response type to class
                    response_type = self.__getattr(response_type)
                    flag = Flag(name="hasError", value=False)
                    metadata = ApiResponseMetadata(flags=[flag])
                    data = {"path": download_path}
                    data["$objectType"] = "pathlib.Path"
                    data["$reserved"] = {}
                    data["$unknownFields"] = {}
                    return response_type(data=data, metadata=metadata)

                # Handle text response
                elif response_data.getheader(name='Content-Type') in ['text/event-stream', 'text/html', 'text/xml', 'text/csv', 'text/javascript', 'text/markdown', 'text/vcard']:
                    # convert response type to class
                    response_type = self.__getattr(response_type)
                    flag = Flag(name="hasError", value=False)
                    metadata = ApiResponseMetadata(flags=[flag])
                    data = response_data.urllib3_response.data
                    return response_type(data=data, metadata=metadata)

                else:
                    # Read the data from original urllib3 response
                    response_data.data = response_data.urllib3_response.data
                    # In the python 3, the response.data is bytes which needs to be decoded to string.
                    if six.PY3:
                        response_data.data = response_data.data.decode('utf-8')

                    return_data = json.loads(response_data.data)
                    return_data = self.__add_header_to_reserved(response_data, return_data, "ETag")
                    if response_type is None and "$objectType" in return_data:
                        response_type = return_data.get("$objectType")
                    if PY2:
                        inner_response_type = response_type.encode('utf-8', 'ignore')
                    else:
                        inner_response_type = response_type

                    return self.deserialize(return_data, inner_response_type)
            else:
                return_data = None
        finally:
            response_data.urllib3_response.drain_conn()
            response_data.urllib3_response.release_conn()
        if _return_http_data_only:
            return (return_data)
        else:
            return (return_data, response_data.status,
                    response_data.getheaders())
    def __sanitize_for_serialization(self, obj):
        """Builds a JSON POST object.

        :param obj: The object to sanitize
        :type obj: :class:`object`
        :return:
        The sanitized object with the following behavior::

            If obj is None, return None.
            If obj is str, int, long, float, bool, return directly.
            If obj is datetime.datetime or datetime.date, convert to string in ISO8601 format.
            If obj is list, sanitize each element in the list.
            If obj is dict, return the dict.
            If obj is a model, return the properties dictionary.
        :rtype: :class:`object`
        """  # noqa: E501
        if obj is None:
            return None
        elif isinstance(obj, self.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [self.__sanitize_for_serialization(sub_obj)
                    for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(self.__sanitize_for_serialization(sub_obj)
                         for sub_obj in obj)
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()
        if isinstance(obj, dict):
            obj_dict = obj
        else:
            # Convert model obj to dict except
            # attributes `swagger_types`, `attribute_map`
            # and attributes which value is not None.
            # Convert attribute name to json key in
            # model definition for request.
            obj_dict = {obj.attribute_map[attr]: getattr(obj, attr)
                        for attr, _ in six.iteritems(obj.swagger_types)
                        if getattr(obj, attr) is not None}
        return {key: self.__sanitize_for_serialization(val)
                for key, val in six.iteritems(obj_dict)}

    def __deserialize_response(self, response, response_type):
        """Deserializes response into an object based on the provided response type.

        :param response: A response object to be deserialized
        :type response: :class:`~ntnx_security_py_client.rest.RESTResponse`, required
        :param response_type: A class literal for deserialized object, or string of class name
        :type response_type: required
        :return: Deserialized object
        :rtype: :class:`object`
        """  # noqa: E501

        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return self.__deserialize_file(response)
        # fetch data from response object
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data
        return self.deserialize(data, response_type)

    def deserialize(self, data, klass, discriminator = None):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str
        :param klass: class literal, or string of class name
        :param discriminator: Deprecated: Discriminator to deserialize into specific model object for a oneOf object. This is no longer needed and will be removed from the method definition soon.
        :type discriminator: :class:`str`
        :return: Deserialized object
        :rtype: :class:`object`
        """  # noqa: E501
        if data is None:
            return None
        if type(klass) == str:
            if klass == 'pathlib.Path':
                data['path'] = Path(data['path'])
                return data
            if klass.startswith('list['):
                sub_kls = re.match(r'list\[(.*)\]', klass).group(1)
                return [self.deserialize(sub_data, sub_kls)
                        for sub_data in data]
            if klass.startswith('dict('):
                sub_kls = re.match(r'dict\(([^,]*), (.*)\)', klass).group(2)
                return {str(k): self.deserialize(v, sub_kls)
                        for k, v in six.iteritems(data)}
            # convert str to class
            if klass in self.NATIVE_TYPES_MAPPING:
                klass = self.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = self.__getattr(klass)
        if klass in self.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass)
        elif klass == object:
            return self.__deserialize_object(data)
        elif klass == datetime.date:
            return self.__deserialize_date(data)
        elif klass == datetime.datetime:
            return self.__deserialize_datatime(data)
        else:
            return self.__deserialize_model(data, klass)

    def _call_api(self, resource_path, method,
                 path_params=None, query_params=None, header_params=None,
                 body=None, post_params=None, files=None,
                 response_type=None, auth_settings=None, async_req=None,
                 _return_http_data_only=None, collection_formats=None,
                 _preload_content=True, _request_timeout=None):
        """Makes the HTTP request (synchronous) and returns deserialized data. To make an asynchronous request, set the async_req to True.

        :param resource_path: Path to API endpoint
        :type resource_path: :class:`str`, required
        :param method: HTTP method name
        :type method: :class:`str`, required
        :param path_params: A dictionary of path parameters in the url
        :type path_params: :class:`dict`
        :param query_params: A list of tuples denoting query parameters in the url
        :type query_params: :class:`list`
        :param header_params: A dictionary of header parameters to be placed in request headers
        :type header_params: :class:`dict`
        :param body: Request body
        :param post_params: Request's form parameters for application/x-www-form-urlencoded or multipart/form-data content type
        :type post_params: :class:`str`
        :param files: A dictionary in {key -> filename, value -> filepath} format for multipart/form-data.
        :type files: :class:`dict`
        :param response_type: Response data type
        :type response_type: :class:`str`
        :param auth_settings: A list of auth settings names for the request. The following values are accepted currently: ::

            ['apiKeyAuthScheme', 'basicAuthScheme']
        :type auth_settings: :class:`list`
        :param async_req: A flag to toggle between synchronous/asynchronous connection for HTTP request (**Default** False)
        :type async_req: :class:`bool`
        :param _return_http_data_only: A flag to retrurn response data without the status code and response headers
        :type _return_http_data_only: :class:`bool`
        :param collection_formats: A dictionary of collection formats for path, query, header, and post parameters
        :type collection_formats: :class:`dict`
        :param _preload_content: If False, the urllib3.HTTPResponse object will be returned without reading/decoding response data (**Default** True)
        :type _preload_content: :class:`bool`
        :param _request_timeout: Timeout setting for this request. It needs to be a pair (list) of
            [connect, read] timeouts. If invalid list length or type provided,
            the timeouts will be derived from the :class:`~ntnx_security_py_client.configuration.Configuration`.
        :type _request_timeout: :class:`list`
        :return: If async_req parameter is True, the request will be called asynchronously and returns the request thread.
            If async_req is False or missing, then the method will return the deserialized response.
        """
        if not async_req:
            return self.__call_api(resource_path, method,
                                   path_params, query_params, header_params,
                                   body, post_params, files,
                                   response_type, auth_settings,
                                   _return_http_data_only, collection_formats,
                                   _preload_content, _request_timeout)
        else:
            self.__initialize_threadpool()
            thread = self.__pool.apply_async(self.__call_api, (resource_path,
                                           method, path_params, query_params,
                                           header_params, body,
                                           post_params, files,
                                           response_type, auth_settings,
                                           _return_http_data_only,
                                           collection_formats,
                                           _preload_content, _request_timeout))
            return thread

    def request(self, method, url, query_params=None, headers=None,
                post_params=None, body=None, _preload_content=True,
                _request_timeout=None):
        client_request_timeout = self.__get_request_timeout(_request_timeout)

        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.GET(url,
                                        query_params=query_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=client_request_timeout,
                                        headers=headers)
        elif method == "HEAD":
            return self.rest_client.HEAD(url,
                                         query_params=query_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=client_request_timeout,
                                         headers=headers)
        elif method == "OPTIONS":
            return self.rest_client.OPTIONS(url,
                                            query_params=query_params,
                                            headers=headers,
                                            post_params=post_params,
                                            _preload_content=_preload_content,
                                            _request_timeout=client_request_timeout,
                                            body=body)
        elif method == "POST":
            return self.rest_client.POST(url,
                                         query_params=query_params,
                                         headers=headers,
                                         post_params=post_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=client_request_timeout,
                                         body=body)
        elif method == "PUT":
            return self.rest_client.PUT(url,
                                        query_params=query_params,
                                        headers=headers,
                                        post_params=post_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=client_request_timeout,
                                        body=body)
        elif method == "PATCH":
            return self.rest_client.PATCH(url,
                                          query_params=query_params,
                                          headers=headers,
                                          post_params=post_params,
                                          _preload_content=_preload_content,
                                          _request_timeout=client_request_timeout,
                                          body=body)
        elif method == "DELETE":
            return self.rest_client.DELETE(url,
                                           query_params=query_params,
                                           headers=headers,
                                           _preload_content=_preload_content,
                                           _request_timeout=client_request_timeout,
                                           body=body)
        else:
            error_message = "http method must be `GET`, `HEAD`, `OPTIONS`, `POST`, `PATCH`, `PUT` or `DELETE`."
            logger.error(error_message)
            raise ValueError(error_message)

    def __parameters_to_tuples(self, params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :type params: :class:`dict`
        :param collection_formats: Parameter collection formats
        :type collection_formats: :class:`dict`
        :returns: Parameters as list of tuples, collections formatted
        """  # noqa: E501

        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in six.iteritems(params) if isinstance(params, dict) else params:  # noqa: E501
            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == 'multi':
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == 'ssv':
                        delimiter = ' '
                    elif collection_format == 'tsv':
                        delimiter = '\t'
                    elif collection_format == 'pipes':
                        delimiter = '|'
                    else:  # csv is the default
                        delimiter = ','
                    new_params.append(
                        (k, delimiter.join(str(value) for value in v)))
            else:
                new_params.append((k, v))
        return new_params

    def __prepare_post_parameters(self, post_params=None, files=None):
        """Builds form parameters.

        :param post_params: Normal form parameters
        :param files: File parameters
        :return: Form parameters with files
        """  # noqa: E501

        params = []
        if post_params:
            params = post_params
        if files:
            for k, v in six.iteritems(files):
                if not v:
                    continue
                file_names = v if type(v) is list else [v]
                for n in file_names:
                    with open(n, 'rb') as f:
                        filename = os.path.basename(f.name)
                        filedata = f.read()
                        mimetype = (mimetypes.guess_type(filename)[0] or
                                    'application/octet-stream')
                        params.append(
                            tuple([k, tuple([filename, filedata, mimetype])]))
        return params

    def _select_header_accept(self, accepts):
        """Returns `Accept` based on an array of headers provided.

        :param accepts: List of headers
        :type accepts: :class:`list`
        :return: If application/json is available, returns that.
            Otherwise, returns a comma separated string of all the provided Accept types
        """  # noqa: E501

        if not accepts:
            return

        accepts = [x.lower() for x in accepts]
        return ', '.join(accepts)

    def _select_header_content_type(self, content_types):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types
        :type content_types: :class:`list`
        :return: Content-Type (e.g. application/json)
        """  # noqa: E501

        if not content_types:
            return 'application/json'
        content_types = [x.lower() for x in content_types]
        if 'application/json' in content_types or '*/*' in content_types:
            return 'application/json'
        else:
            return content_types[0]

    def __update_params_for_auth(self, headers, queries, auth_settings):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param queries: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        """  # noqa: E501
        if not auth_settings:
            return

        for auth in auth_settings:
            auth_setting = self.configuration._auth_settings().get(auth)
            if auth_setting:
                headers.pop(auth_setting['key'], None)

                if not auth_setting['value']:
                    continue
                elif auth_setting['in'] == 'header':
                    headers[auth_setting['key']] = auth_setting['value']
                elif auth_setting['in'] == 'query':
                    queries.append((auth_setting['key'], auth_setting['value']))
                else:
                    error_message = 'Authentication token must be in `query` or `header`'
                    logger.error(error_message)
                    raise ValueError(error_message)

    @staticmethod
    def get_etag(object):
        """Method to extract ETag from an object if exists

        The ETag is usually provided in the response of the GET API calls which can further be used in other HTTP operations.

        :param object: The object from which etag needs to be fetched
        :type object: :class:`object`
        :return: ETag header value for the object
        :rtype: :class:`str` or :class:`None`
        """  # noqa: E501
        etag = None
        if object:
            if hasattr(object, "get_reserved") and 'ETag' in object.get_reserved():
                etag = ApiClient.get_value_for_case_insensitive_key_match(object.get_reserved(), 'ETag')

            if not etag and hasattr(object, "data") and hasattr(object.data, "get_reserved") and 'ETag' in object.data.get_reserved():
                etag = ApiClient.get_value_for_case_insensitive_key_match(object.data.get_reserved(), 'ETag')

        return etag

    @staticmethod
    def get_value_for_case_insensitive_key_match(object, key):
        if object and key:
            key = key.lower()
            for k in object:
                if k.lower() == key:
                    return object[k]

    def __add_header_to_reserved(self, response_data, return_data, key):
        if hasattr(response_data, "getheaders") and key in response_data.getheaders():
            header_val = response_data.getheader(key)
            if "data" in return_data.keys():
                if isinstance(return_data["data"], list) and len(return_data["data"])>0:
                    for item in return_data["data"]:
                        if isinstance(item, dict) and "$reserved" in item.keys():
                            item["$reserved"][key] = header_val
                elif isinstance(return_data["data"], dict) and "$reserved" in return_data["data"].keys():
                    return_data["data"]["$reserved"][key] = header_val
        return return_data

    def __deserialize_file(self, response):
        """Deserializes body to file

        Saves response body into a file in a temporary folder, using the filename from the `Content-Disposition` header if provided.

        :param response: :class:`~ntnx_security_py_client.rest.RESTResponse`
        :return: File path
        """

        path = ''
        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"*]+)[\'"]?', content_disposition).group(1)
            path = os.path.join(self.configuration.download_directory, filename)
        else:
            fd, path = tempfile.mkstemp(dir=self.configuration.download_directory)
            os.close(fd)

        path_tup = os.path.splitext(path)
        timestring = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3]
        path = "%s_%s%s" % (path_tup[0], timestring, path_tup[1])
        logger.info('Writing response data to file at %s' % path)
        with open(path, "wb") as f:
            if isinstance(response.urllib3_response, urllib3.response.HTTPResponse):
                for chunk in response.urllib3_response.stream(self.configuration.download_chunk_size):
                    f.write(chunk)

        return path

    def __deserialize_primitive(self, data, klass):
        """Deserializes string to primitive type.

        :param data:
        :type data: :class:`str`
        :param klass: class literal
        :return: :class:`int` or :class:`long` or :class:`float` or :class:`str` or :class:`bool` value
        """

        try:
            return klass(data)
        except UnicodeEncodeError:
            return six.text_type(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return a original value.
        :return: :class:`object`
        """
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.
        :param string:
        :type string: str
        :raises :class:`~ntnx_security_py_client.rest.ApiException` if provided string is not a valid date
        :return: date object
        """

        if type(string) == datetime.date:
            return string

        try:
            from dateutil.parser import parse
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason="Failed to parse `{0}` as date object".format(string)
            )

    def __deserialize_datatime(self, string):
        """Deserializes string to datetime.

        The string should be in ISO8601 datetime format.

        :param string:
        :type string: :class:`str`
        :raises :class:`~ntnx_security_py_client.rest.ApiException` if provided string is not a valid datetime
        :return: :class:`datetime`
        """

        if type(string) == datetime.datetime:
            return string

        try:
            from dateutil.parser import parse
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason=(
                    "Failed to parse `{0}` as datetime object"
                    .format(string)
                )
            )

    def __hasattr(self, object, name):
            return name in object.__class__.__dict__

    def __getattr(self, name):
        if name in self.DUPLICATE_SCHEMA_MAPPING:
            return getattr(ntnx_security_py_client.models, self.DUPLICATE_SCHEMA_MAPPING[name])
        if hasattr(ntnx_security_py_client.models, name.split('.')[-1]):
            return getattr(ntnx_security_py_client.models, name.split('.')[-1])
        else:
            return getattr(ntnx_security_py_client.models, name)

    def __deserialize_model(self, data, klass):
        """Deserializes list or dict to model.
        :param data:
        :type data: :class:`dict`, :class:`list`
        :param klass: class literal
        :return: Model object
        """

        if not hasattr(klass, 'swagger_types') and not self.__hasattr(klass, 'get_real_child_model'):
            return data
        kwargs = {}
        if klass.swagger_types is not None:
            for attr, attr_type in six.iteritems(klass.swagger_types):
                if data is not None and isinstance(data, (list, dict)):
                    value = None
                    if klass.attribute_map[attr] in data:
                        value = data[klass.attribute_map[attr]]
                        data.pop(klass.attribute_map[attr])
                    elif attr in data:
                        value = data[attr]
                        data.pop(attr)

                    one_of_value = None
                    if attr_type.startswith('OneOf'):
                        if type(value) is list:
                            one_of_value = []
                            # OneOf of list type can have list of objects or list of primitive types
                            for value_item in value:
                                if type(value_item) is not dict:
                                    one_of_value.append(self.deserialize(value_item, type(value_item)))
                                else:
                                    one_of_value.append(self.deserialize(value_item, self.__getattr(value_item['$objectType'])))
                        elif type(value) is dict:
                            # OneOf of dict type can be a single response object or a map of primitive types
                            if '$objectType' in value:
                                one_of_value = self.deserialize(value, value['$objectType']
                                    if value['$objectType'] == 'pathlib.Path' else self.__getattr(value['$objectType']))
                            else:
                                one_of_value = {}
                                for item_key, item_value in six.iteritems(value):
                                    one_of_value[item_key] = self.deserialize(item_value, type(item_value))
                        else:
                            one_of_value = value

                    kwargs[attr] = self.deserialize(value, attr_type) if one_of_value is None else one_of_value

            if "_unknown_fields" in klass.swagger_types:
                if "_unknown_fields" not in kwargs or kwargs["_unknown_fields"] is None:
                    kwargs["_unknown_fields"] = {}
                for key, value in data.items():
                    if value is not None:
                        kwargs["_unknown_fields"][key] = value

        instance = klass(**kwargs)
        if (isinstance(instance, dict) and
                klass.swagger_types is not None and
                isinstance(data, dict)):
            for key, value in data.items():
                if key not in klass.swagger_types:
                    instance[key] = value

        return instance



# coding: utf-8


"""
IGNORE:
    Nutanix Security APIs

    Manage security features, such as encryption, certificates, or platform hardening.  # noqa: E501

    OpenAPI spec version: 4.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_security_py_client.models.security.v4.report.Severity import Severity  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.Stig import Stig  # noqa: F401,E501
from ntnx_security_py_client.models.security.v4.report.StigStatus import StigStatus  # noqa: F401,E501

class StigProjection(Stig):

    """

    :param title: (:attr:`title`) Title of the STIG control.
    :type title: 
     max=199 chars
    
    :param rule_id: (:attr:`rule_id`) Rule ID of the STIG control.
    :type rule_id: 
    
    
    :param stig_version: (:attr:`stig_version`) STIG ID of the control.
    :type stig_version: 
    
    
    :param identifiers: (:attr:`identifiers`) Additional identifiers used to describe this control.
    :type identifiers: 
    
    
    :param affected_clusters: (:attr:`affected_clusters`) List of clusters that failed the STIG control.
    :type affected_clusters: 
    
    
    :param severity: (:attr:`severity`) 
    :type severity: 
    
    
    :param status: (:attr:`status`) 
    :type status: 
    
    
    :param comments: (:attr:`comments`) The comments to explain why a STIG rule applies or does not apply to the cluster.
    :type comments: 
     max=499 chars
    
    :param fix_text: (:attr:`fix_text`) The command/steps to fix the STIG rule failure.
    :type fix_text: 
     max=499 chars
    
    :param benchmark_id: (:attr:`benchmark_id`) Benchmark ID of the STIG rules.
    :type benchmark_id: 
    
    
    :param ext_id: (:attr:`ext_id`) A globally unique identifier of an instance that is suitable for external consumption. 
    :type ext_id: 
    
    
    :param links: (:attr:`links`) A HATEOAS style link for the response.  Each link contains a user-friendly name identifying the link and an address for retrieving the particular resource. 
    :type links: 
    
    
    :param tenant_id: (:attr:`tenant_id`) A globally unique identifier that represents the tenant that owns this entity. The system automatically assigns it, and it and is immutable from an API consumer perspective (some use cases may cause this ID to change - For instance, a use case may require the transfer of ownership of the entity, but these cases are handled automatically on the server). 
    :type tenant_id: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'title': 'str',
        'rule_id': 'str',
        'stig_version': 'str',
        'identifiers': 'list[str]',
        'affected_clusters': 'list[str]',
        'severity': 'security.v4.report.Severity',
        'status': 'security.v4.report.StigStatus',
        'comments': 'str',
        'fix_text': 'str',
        'benchmark_id': 'str',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'title': 'title',
        'rule_id': 'ruleId',
        'stig_version': 'stigVersion',
        'identifiers': 'identifiers',
        'affected_clusters': 'affectedClusters',
        'severity': 'severity',
        'status': 'status',
        'comments': 'comments',
        'fix_text': 'fixText',
        'benchmark_id': 'benchmarkId',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, title=None, rule_id=None, stig_version=None, identifiers=None, affected_clusters=None, severity=None, status=None, comments=None, fix_text=None, benchmark_id=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        Stig.__init__(self, title, rule_id, stig_version, identifiers, affected_clusters, severity, status, comments, fix_text, benchmark_id, ext_id, links, tenant_id, *args, **kwargs)
        self.discriminator = None

    def _initialize_object_type(self):
        return 'security.v4.report.StigProjection'

    def _initialize_object_version(self):
        return 'v4.r0'


    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(StigProjection, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StigProjection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other

